unit ch02;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, OpenGL;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }

  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  hrc: HGLRC;  // Permanent Rendering Context

  f_Hdc : LongInt;

  implementation

{$R *.DFM}

procedure InitGL;	// All Setup For OpenGL Goes Here
begin
// select clearing color
   glClearColor( 0, 0, 0, 0);
// initialize viewing values
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   glOrtho( 0, 1, 0, 1, -1, 1);
end;

procedure DrawGLScene();
 // Here's Where We Do All The Drawing!!!
 //Right after glLoadIdentity and before DrawGLScene:=true
begin
  // Enable depth testing and clear the color and depth buffers.
  glEnable(GL_DEPTH_TEST);
  // Clear The Screen And The Depth Buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);  // Select The Projection Matrix
  glLoadIdentity();    // Reset The View
  glOrtho( 0, 1, 0, 1, -1, 1);
  // Set the drawing color to blue.
  glColor3f(1.0, 1.0, 1.0);
  glBegin (GL_POLYGON);
      glVertex3f (0.25, 0.25, 0);
      glVertex3f (0.75, 0.25, 0);
      glVertex3f (0.75, 0.75, 0);
      glVertex3f (0.25, 0.75, 0);
   glEnd;
  // Flush the drawing pipeline.
  glFlush;
end;

procedure CleanUp(Handle: HDC); //Properly Kill The Window
begin
  if hrc<> 0 then       //Is There A Rendering Context?
    begin
       //Are We Able To Release Dc and Rc contexts?
       if (not wglMakeCurrent(handle,0)) then
         MessageBox(0,'Release of DC and RC failed.'
                    ,' Shutdown Error',MB_OK or MB_ICONERROR);
      //Are We Able To Delete The Rc?
       if (not wglDeleteContext(hRc)) then
        begin
          MessageBox(0,'Release of Rendering Context failed.',
                       ' Shutdown Error',MB_OK or MB_ICONERROR);
          hRc:=0;   //Set Rc To Null
        end;
    end;
end;

procedure SetDCPixelFormat(Handle: HDC;ColorBits,DepthBufferBits:integer);
var
  pfd: TPixelFormatDescriptor;
  nPixelFormat: Integer;

begin
  FillChar(pfd, SizeOf(pfd), 0);

  with pfd do begin
    nSize     := sizeof(pfd);             // Size of this structure
    nVersion  := 1;                       // Version number
    dwFlags   := PFD_SUPPORT_OPENGL Or PFD_DRAW_TO_WINDOW
                 Or  PFD_TYPE_RGBA;       // Flags
    iPixelType:= PFD_TYPE_RGBA;           // RGBA pixel values
    cColorBits:= ColorBits;               // 24-bit color
    cDepthBits:= DepthBufferBits;         // 32-bit depth buffer
    iLayerType:= PFD_MAIN_PLANE;          // Layer type
  end;

  nPixelFormat := ChoosePixelFormat(Handle, @pfd);
  //Did We Find A Matching Pixelformat?
  if (nPixelFormat=0) then
    begin
    CleanUp(handle);                          //Reset The Display
      MessageBox(0,'Cant''t Find A Suitable PixelFormat.'
                      ,'Error',MB_OK or MB_ICONEXCLAMATION);
      Halt(1); { Halt right here! }
    end;

  //Are We Able To Set The Pixelformat?
  if (not SetPixelFormat(Handle, nPixelFormat, @pfd)) then
    begin
      CleanUp(handle);                          //Reset The Display
      MessageBox(0,'Cant''t set PixelFormat.'
              ,'Error',MB_OK or MB_ICONEXCLAMATION);
      Halt(1); { Halt right here! }
    end;

  hrc := wglCreateContext(Handle);
  if (hRc=0) then
    begin
    CleanUp(handle);                        //Reset The Display
      MessageBox(0,'Cant''t create a GL rendering context.'
                         ,'Error',MB_OK or MB_ICONEXCLAMATION);
      Halt(1); { Halt right here! }
    end;

  //Are We Able To Activate The Rendering Context?
  if (not wglMakeCurrent(Handle, hrc)) then
    begin
    CleanUp(handle);                      //Reset The Display
    MessageBox(0,'Cant''t activate the GL rendering context.'
                          ,'Error',MB_OK or MB_ICONEXCLAMATION);
      Halt(1); { Halt right here! }
    end;

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  f_Hdc:=GetDC(Handle);
  // Create a rendering context.
   SetDCPixelFormat(f_Hdc,16,16);
  InitGL;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  // Clean up and terminate.
  CleanUp(f_Hdc);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  // Draw the scene.
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
  DrawGLScene;
end;

procedure TForm1.FormResize(Sender: TObject);
 begin
  // Redefine the viewing volume and viewport
  //when the window size changes.
   wglMakeCurrent(f_Hdc, hrc);
   // Reset The Current Viewport And Perspective
   // Now, set up the viewing area-select the full client area
   glViewport( 0, 0, Width , Height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();  // Reset The Projection Matrix
   gluOrtho2D(0, Width, 0, Height);
   DrawGLScene();
 end;
end.

