unit Ch07_01;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, OpenGL, SPF, Math, ExtCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormResize(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }

  end;

var
  Form1: TForm1;

  position : array[0..3] of GLfloat = (0,0,1.5,1);
  spin : GLfloat;
  mat_specular : array[0..3] of GLfloat = (1,1,1,1);
  diffuseMaterial : array[0..3] of GLfloat = (0.5, 0.5, 0.5, 1);

  f_Hdc : LongInt;

implementation

{$R *.DFM}

{ doughnut:
 * draws a doughnut, centered at (0, 0, 0) whose axis is aligned with
 * the z-axis.  The doughnut's major radius is R, and minor radius is r.}
procedure doughnut( R1 , R2 : GLdouble;
                    nsides , rings : GLint;
                    thetype : GLenum);
var
   i , j : Integer;
   theta , phi , theta1 , phi1 : GLdouble ;
   p0 , p1 , p2 , p3 :  array[0..2] of GLdouble;
   n0, n1, n2,n3 :  array[0..2] of GLdouble;

begin
    for i := 0 To rings - 1  do
      begin
        theta := i * 2 * 3.1415927 / rings ;
        theta1 := (i + 1) * 2 * 3.1415927 / rings ;
        for j := 0 To nsides - 1  do
          begin
            phi := j * 2 * 3.1415927 / nsides;
            phi1 := (j + 1) * 2 * 3.1415927 / nsides;
            p0[0] := Cos(theta) * (R2 + R1 * Cos(phi));
            p0[1] := -Sin(theta) * (R2 + R1 * Cos(phi));
            p0[2] := R1 * Sin(phi);
            p1[0] := Cos(theta1) * (R2 + R1 * Cos(phi));
            p1[1] := -Sin(theta1) * (R2 + R1 * Cos(phi));
            p1[2] := R1 * Sin(phi);
            p2[0] := Cos(theta1) * (R2 + R1 * Cos(phi1));
            p2[1] := -Sin(theta1) * (R2 + R1 * Cos(phi1));
            p2[2] := R1 * Sin(phi1);
            p3[0] := Cos(theta) * (R2 + R1 * Cos(phi1));
            p3[1] := -Sin(theta) * (R2 + R1 * Cos(phi1));
            p3[2] := R1 * Sin(phi1);
            n0[0] := Cos(theta) * Cos(phi);
            n0[1] := -Sin(theta) * Cos(phi);
            n0[2] := Sin(phi);
            n1[0] := Cos(theta1) * Cos(phi);
            n1[1] := -Sin(theta1) * Cos(phi);
            n1[2] := Sin(phi);
            n2[0] := Cos(theta1) * Cos(phi1);
            n2[1] := -Sin(theta1) * Cos(phi1);
            n2[2] := Sin(phi1);
            n3[0] := Cos(theta) * Cos(phi1);
            n3[1] := -Sin(theta) * Cos(phi1);
            n3[2] := Sin(phi1);
            glBegin (thetype);
                glNormal3dv (@n3);
                    glVertex3dv (@p3);
                glNormal3dv (@n2);
                    glVertex3dv (@p2);
                glNormal3dv (@n1);
                    glVertex3dv (@p1);
                glNormal3dv (@n0);
                    glVertex3dv (@p0);
            glEnd;
        end;
    end;

End;

procedure auxSolidTorus(innerRadius , outerRadius : GLdouble);
begin
//         Another mostly-direct port.
        doughnut(innerRadius, outerRadius, 8, 15, GL_QUADS);
end;

procedure initGL();
begin
//  Initialize material property, light source, lighting model,
//  and depth buffer.

   glClearColor (1, 1, 1, 0);
   glShadeModel (GL_SMOOTH);
   glEnable (GL_LIGHTING);
   glEnable (GL_LIGHT0);
   glMaterialfv (GL_FRONT, GL_DIFFUSE, @diffuseMaterial);
   glMaterialfv (GL_FRONT, GL_SPECULAR, @mat_specular);
   glMaterialf (GL_FRONT, GL_SHININESS, 25);
   glColorMaterial (GL_FRONT, GL_DIFFUSE);
   glEnable (GL_COLOR_MATERIAL);
   glEnable (GL_DEPTH_TEST);
End ;

procedure DrawGLScene();
 begin
   glClear (GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT);
   glPushMatrix;
   gluLookAt (0, 0, 5, 0, 0, 0, 0, 1, 0);

   glPushMatrix;
   glRotated (spin, 1, 0, 0);
   glLightfv (GL_LIGHT0, GL_POSITION, @position);

   glTranslated (0, 0, 1.5);
   glDisable (GL_LIGHTING);
   //glColor3f (1, 0, 0);
    auxSolidTorus(0.1, 0.1);
   glEnable (GL_LIGHTING);
   glPopMatrix;
    auxSolidTorus(0.275, 0.85);
   glPopMatrix;

   //glFlush;
   SwapBuffers (f_Hdc);
End;

procedure TForm1.FormResize(Sender: TObject);
begin
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
// Prevent A Divide By Zero If The Window Is Too Small
// By Making The Height One
  if (Height=0) then   Height:=1;
    glViewport (0, 0, width, height);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity;
   gluPerspective (40, width / height, 1, 20);
   glMatrixMode (GL_MODELVIEW);
   glLoadIdentity;
   InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  spin := spin + 5 ;

  diffuseMaterial[0] := diffuseMaterial[0] + 0.01 ;
  diffuseMaterial[1] := diffuseMaterial[1] + 0.02 ;
  diffuseMaterial[2] := diffuseMaterial[2] - 0.03 ;

  If (diffuseMaterial[0] > 1) Then
    begin
      diffuseMaterial[0] := 0;
      diffuseMaterial[1] := 0.1;
   // if (diffuseMaterial[2] > .5) Then diffuseMaterial[2] := 0.2;
    end;
  glColor4fv (@diffuseMaterial);

  Application.ProcessMessages; //DoEvents
  InvalidateRect(Handle, nil, False);//   DrawGLScene();
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL;
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
 DrawGLScene();
end;

end.
