unit Ch08;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
   Forms, Dialogs, OpenGL, SPF, ExtCtrls ;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    Image1: TImage;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
var
 f_Hdc : LongInt;

procedure glGenTextures(n: GLsizei;
         var textures: GLuint); stdcall; external opengl32;
procedure glBindTexture(target: GLenum;
         texture: GLuint); stdcall; external opengl32;

// The gluBuild2DMipmaps declaration in the Delphi 4 version
//of the OpenGL unit was improperly declared. I've redeclared it here:
function gluBuild2DMipmaps(target: GLenum;
            components, width, height: GLint;
            format,atype: GLenum;
            Data: Pointer): GLint; stdcall; external glu32;
var
  xrot: GLfloat;    // X Rotation
  yrot: GLfloat;    // Y Rotation
const
  z: GLfloat = -5.0;  // Depth Into The Screen
  LightAmbient: array [0..3] of GLfloat = ( 0.5, 0.5, 0.5, 1.0 );
  LightDiffuse: array [0..3] of GLfloat = ( 1.0, 1.0, 1.0, 1.0 );
  LightPosition: array [0..3] of GLfloat = ( 0.0, 0.0, 2.0, 1.0 );
  filter: GLuint = 0;  // Which Filter To Use
var
  texture: array [0..2] of GLuint;    // Storage for 3 textures

procedure getRGB(num : LongInt; var r , g , b : Integer);
begin
//  extract R,G,B from a long format RBG
        b := Trunc((num And 16711680)/ 65536) ;
        g := Trunc((num And 65280)/ 256 );
        r :=  num  And 255 ;
End;

// Load Bitmaps And Convert To Textures
procedure LoadGLTextures (pctPicToLoad : TImage );
 var
  x , y : LongInt;
   c , bitmapWidth, bitmapHeight : LongInt ;
   Red , Green , Blue : Integer ;
   texture1: array[0..63,0..63,0..2] of  GLubyte;
begin
   // Load Texture
    pctPicToLoad.AutoSize := True ;
    bitmapHeight := pctPicToLoad.Height ;  // Set the array
    bitmapWidth := pctPicToLoad.Width ; // size.

 For x := 0 To  bitmapWidth-1  do
     For y := 0 To bitmapHeight-1  do
       begin
         c := ColorToRGB(pctPicToLoad.Canvas.Pixels[x,y]);
         getRGB(c,Red,Green,Blue);
         texture1[ x, bitmapHeight - y - 1,0] := Red ;   //GetRed
         texture1[ x, bitmapHeight - y - 1,1] := Green ; //GetGreen
         texture1[ x, bitmapHeight - y - 1,2] := Blue ;  //GetBlue
       end;

       pctPicToLoad.Visible:=false;

	if not Assigned( @texture1 ) then Halt(1);

	// Create Nearest Filtered Texture
    	glGenTextures(3, texture[0]);
  	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, bitmapWidth, bitmapHeight,
               0, GL_RGB, GL_UNSIGNED_BYTE, @texture1);

	// Create Linear Filtered Texture
	glBindTexture(GL_TEXTURE_2D, texture[1]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, bitmapWidth, bitmapHeight ,
            0, GL_RGB, GL_UNSIGNED_BYTE, @texture1);

	// Create MipMapped Texture
	glBindTexture(GL_TEXTURE_2D, texture[2]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
             GL_LINEAR_MIPMAP_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 3, bitmapWidth, bitmapHeight,
                GL_RGB, GL_UNSIGNED_BYTE, @texture1);
end;

// This Will Be Called Right After The GL Window Is Created
procedure InitGL(Width: GLsizei; Height: GLsizei);
  begin
  	LoadGLTextures(form1.Image1);	        // Load The Texture(s)
    	glEnable(GL_TEXTURE_2D);    	// Enable Texture Mapping
        // This Will Clear The Background Color To White
	glClearColor(1.0, 1.0, 1.0, 0.0);
	glClearDepth(1.0);	     // Enables Clearing Of The Depth Buffer
	glDepthFunc(GL_LESS);	    // The Type Of Depth Test To Do
	glEnable(GL_DEPTH_TEST);    // Enables Depth Testing
	glShadeModel(GL_SMOOTH);   // Enables Smooth Color Shading

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();  // Reset The Projection Matrix

        // Calculate The Aspect Ratio Of The Window
      	gluPerspective(45.0,Width/Height,0.1,100.0);

	glMatrixMode(GL_MODELVIEW);

	glLightfv(GL_LIGHT1, GL_AMBIENT, @LightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, @LightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION,@LightPosition);
	glEnable(GL_LIGHT1);
end;

procedure DrawGLScene();

  begin
        // Clear The Screen And The Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();   // Reset The View
	glTranslatef(0.0,0.0,z);

	glRotatef(xrot,1.0,0.0,0.0);
	glRotatef(yrot,0.0,1.0,0.0);

  	glBindTexture(GL_TEXTURE_2D, texture[filter]);

	glBegin(GL_QUADS);
		// Front Face
		glNormal3f( 0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
		// Back Face
		glNormal3f( 0.0, 0.0,-1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
		// Top Face
		glNormal3f( 0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0,  1.0,  1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0,  1.0,  1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
		// Bottom Face
		glNormal3f( 0.0,-1.0, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f( 1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
		// Right face
		glNormal3f( 1.0, 0.0, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
		// Left Face
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
	glEnd();

        swapBuffers(f_Hdc);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
   wglMakeCurrent(f_Hdc,hrc); //activate the RC
   // Prevent A Divide By Zero If The Window Is Too Small
   if (Height=0) then   Height:=1;
   // Reset The Current Viewport And Perspective Transformation
   glViewport(0, 0, Width, Height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(45.0,Width/Height,0.1,100.0);
   glMatrixMode(GL_MODELVIEW);
   InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL(width,height);
{
 // loading the texture from file
  if  OpenPictureDialog1.Execute  then
    begin
      Image1.AutoSize := True;
      Image1.Picture.LoadFromFile (OpenPictureDialog1.FileName);
    end
  else
      halt(1);
}
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 DrawGLScene();
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
        z:=z-0.05 ;
        if z <= -10 then
        begin
         z:=-5;
         filter:=filter+1;
         if (filter>2) then filter:=0;
        end;
  	xrot := xrot + 3 ;
	yrot := yrot + 3 ;
  Application.ProcessMessages; //DoEvents
  InvalidateRect(Handle, nil, False);//   DrawGLScene();
 end;
end.


