unit Ch10;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
   Forms, Dialogs, OpenGL, SPF, ExtCtrls ;

type
  TForm1 = class(TForm)
    Image1: TImage;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  f_Hdc : LongInt;

implementation

{$R *.DFM}

procedure glGenTextures(n: GLsizei; var textures: GLuint);
stdcall; external 'opengl32.dll';
procedure glBindTexture(target: GLenum; texture: GLuint);
stdcall; external 'opengl32.dll';

// The gluBuild2DMipmaps declaration in the
//Delphi 4 version of the OpenGL unit
// was improperly declared. I've redeclared it here:=
function gluBuild2DMipmaps(target: GLenum; components,
 width, height: GLint; format, atype: GLenum; Data: Pointer): GLint;
 stdcall; external glu32;

var

  light : bool;				// Lighting ON/OFF

  part1 : GLuint; 			// Start Of Disc
  part2 : GLuint; 			// End Of Disc
  p1 : GLuint = 0;			// Increase 1
  p2 : GLuint = 1;			// Increase 2

  xrot : GLfloat;		      // X Rotation
  yrot : GLfloat;		      // Y Rotation
  xspeed : GLfloat;		      // X Rotation Speed
  yspeed : GLfloat;		      // Y Rotation Speed
  z : GLfloat = -5.0;		      // Depth Into The Screen

  quadratic : GLUquadricObj;	     // Storage For Our Quadratic Objects

  LightAmbient : array [0..3] of GLfloat = (0.5, 0.5, 0.5, 1.0);
  LightDiffuse : array [0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
  LightPosition : array [0..3] of GLfloat = (0.0, 0.0, 2.0, 1.0);

  filter : GLuint;				// Which Filter To Use
  texture : array [0..2] of GLuint;		// Storage For 3 Textures
  selectedobject : GLuint = 0;			// Which Object To Draw


procedure getRGB(num : LongInt; var r , g , b : Integer);
begin
//  extract R,G,B from a long format RBG
        b := Trunc((num And 16711680)/ 65536) ;
        g := Trunc((num And 65280)/ 256 );
        r :=  num  And 255 ;
End;

// Load Bitmaps And Convert To Textures
function LoadGLTextures (pctPicToLoad : TImage): boolean;
var
  Status : boolean;                    // Status Indicator
  x , y : LongInt;
  c ,bitmapWidth, bitmapHeight : LongInt ;
  Red , Green , Blue : Integer ;
  TextureImage: array[0..63,0..63,0..2] of  GLubyte;
begin
  Status := FALSE;
   // create the Texture
    pctPicToLoad.AutoSize := True ;
    bitmapHeight := pctPicToLoad.Height ;  // Set the array
    bitmapWidth := pctPicToLoad.Width ; // size.

 For x := 0 To  bitmapWidth-1  do
     For y := 0 To bitmapHeight-1  do
       begin
         c := ColorToRGB(pctPicToLoad.Canvas.Pixels[x,y]);
         getRGB(c,Red,Green,Blue);
         TextureImage[ x, bitmapHeight - y - 1,0] := red ;
         TextureImage[ x, bitmapHeight - y - 1,1] := green ;
         TextureImage[ x, bitmapHeight - y - 1,2] := blue ;
       end;
       pctPicToLoad.Visible:=false;


  if (@TextureImage <> nil) then
  begin
    Status := TRUE;                         // Set The Status To TRUE
    glGenTextures(3, texture[0]);          // Create Two Textures

  // Create Nearest Filtered Texture
  glGenTextures(3, texture[0]);
  glBindTexture(GL_TEXTURE_2D, texture[0]);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, bitmapWidth, bitmapHeight,
  0, GL_RGB, GL_UNSIGNED_BYTE, @TextureImage);

  // Create Linear Filtered Texture
  glBindTexture(GL_TEXTURE_2D, texture[1]);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, bitmapWidth, bitmapHeight,
  0, GL_RGB, GL_UNSIGNED_BYTE, @TextureImage);

  // Create MipMapped Texture
  glBindTexture(GL_TEXTURE_2D, texture[2]);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
                  GL_LINEAR_MIPMAP_NEAREST);
  gluBuild2DMipmaps(GL_TEXTURE_2D, 3, bitmapWidth, bitmapHeight,
   GL_RGB, GL_UNSIGNED_BYTE, @TextureImage);
  end;

  result := Status;                     // Return The Status
end;

function InitGL : boolean;  // All Setup For OpenGL Goes Here
begin
  // Jump To Texture Loading Routine
  if (LoadGLTextures(form1.image1) = false) then
  begin
    result := FALSE;	 // If Texture Didn't Load Return FALSE
    exit;
  end;

  glEnable(GL_TEXTURE_2D); 		// Enable Texture Mapping
  glShadeModel(GL_SMOOTH);	        // Enable Smooth Shading
  glClearColor(1.0, 1.0, 1.0, 0.5);	// White Background
  glClearDepth(1.0);			// Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);		// Enables Depth Testing
  glDepthFunc(GL_LEQUAL);		// The Type Of Depth Testing To Do
  // Really Nice Perspective Calculations
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  glLightfv(GL_LIGHT1, GL_AMBIENT, @LightAmbient);  // Setup The Ambient Light
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @LightDiffuse);  // Setup The Diffuse Light
  glLightfv(GL_LIGHT1, GL_POSITION,@LightPosition); // Position The Light
  glEnable(GL_LIGHT1);				    // Enable Light One
  // Create A Pointer To The Quadric Object (Return 0 If No Memory)
  quadratic := gluNewQuadric();
  gluQuadricNormals(quadratic, GLU_SMOOTH);   // Create Smooth Normals
  gluQuadricTexture(quadratic, GL_TRUE);     // Create Texture Coords

  result := TRUE;					// Initialization Went OK
end;

procedure glDrawCube;
begin
  glBegin(GL_QUADS);
    // Front Face
    glNormal3f( 0.0, 0.0, 1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
    // Back Face
    glNormal3f( 0.0, 0.0,-1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
    // Top Face
    glNormal3f( 0.0, 1.0, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0,  1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    // Bottom Face
    glNormal3f( 0.0,-1.0, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    // Right Face
    glNormal3f( 1.0, 0.0, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    // Left Face
    glNormal3f(-1.0, 0.0, 0.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
  glEnd();
end;

function DrawGLScene : boolean;	 // Here's Where We Do All The Drawing
begin
  // Clear The Screen And The Depth Buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity(); 		// Reset The View
  glTranslatef(0.0,0.0,z);

  glRotatef(xrot,1.0,0.0,0.0);
  glRotatef(yrot,0.0,1.0,0.0);

  glBindTexture(GL_TEXTURE_2D, texture[filter]);

  case selectedobject of
    0:    glDrawCube();
    1:    begin
            glTranslatef(0.0,0.0,-1.5);		// Center The Cylinder
            // A Cylinder With A Radius Of 0.5 And A Height Of 2
            gluCylinder(quadratic,1.0,1.0,3.0,32,32);
          end;
          // Draw A Disc (CD Shape) With An Inner Radius Of 0.5,
          // And An Outer Radius Of 2.  Plus A Lot Of Segments ;)
    2:    gluDisk(quadratic,0.5,1.5,32,32);
          // Draw A Sphere With A Radius Of 1 And 16 Longitude
          // And 16 Latitude Segments
    3:    gluSphere(quadratic,1.3,32,32);
    4:    begin
            glTranslatef(0.0,0.0,-1.5);	  	// Center The Cone
            // A Cone With A Bottom Radius Of .5 And A Height Of 2
            gluCylinder(quadratic,1.0,0.0,3.0,32,32);
          end;
    5:    begin
            part1 := part1 + p1;
            part2 := part2 + p2;

            if (part1>359) then				// 360 Degrees
            begin
              p1:=0;
              part1:=0;
              p2:=1;
              part2:=0;
            end;
            if (part2>359) then	 	// 360 Degrees
            begin
              p1:=1;
              p2:=0;
            end;
            // A Disk Like The One Before
            gluPartialDisk(quadratic,0.5,1.5,32,32,part1,part2-part1);
          end;
  end;

  xrot := xrot + xspeed;
  yrot := yrot + yspeed;         // Increase The Second Counter

  swapBuffers(f_Hdc);

  result := TRUE;				// Everything Went OK
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
  // Prevent A Divide By Zero If The Window Is Too Small
  if (Height=0)
    then Height:=1;

  glViewport(0,0,width,height);		// Reset The Current Viewport
  glMatrixMode(GL_PROJECTION);		// Select The Projection Matrix
  glLoadIdentity();			// Reset The Projection Matrix
  // Calculate Window Aspect Ratio
  gluPerspective(45.0,Width/Height,0.1,100.0);
  glMatrixMode(GL_MODELVIEW);	  // Select The Modelview Matrix
  glLoadIdentity();		  // Reset The Modelview Matrix
  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL();
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 DrawGLScene();
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

    if key=ord('L')  then
    begin
      light := not(light);
      if light then
        glEnable(GL_LIGHTING)
      else
        glDisable(GL_LIGHTING);
    end;

    if key=ord('F')  then
    begin
      filter := filter + 1;
      if (filter>2) then filter := 0;
    end;

    if key=32  then
    begin
      selectedobject := selectedobject + 1;
      if (selectedobject>5) then selectedobject := 0;
    end;

    if (key=39) then xspeed := xspeed - 0.1;
    if (key=37) then xspeed := xspeed + 0.1;
    if (key=38) then yspeed := yspeed + 0.1;
    if (key=40) then yspeed := yspeed - 0.1;

  Application.ProcessMessages; //DoEvents
  InvalidateRect(Handle, nil, False);//   DrawGLScene();
end;

end.
