unit Ch12;
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics,
   Controls, Forms, Dialogs, OpenGL, SPF;

type
  TForm1 = class(TForm)
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
var
  Form1: TForm1;

  f_Hdc : LongInt;

implementation
{$R *.DFM}

type TPoints= array[0..3,0..2] of GLfloat;
var
  ctrlpoints1 : TPoints = (
        ( -4.0, -4.0, 0.0), ( -2.0, 4.0, 0.0),
        (2.0, -4.0, 0.0), (4.0, 4.0, 0.0));

  ctrlpoints2 : TPoints = (
        ( 1.0, 1.0, 0.0), ( 2.0, 3.0, 0.0),
        (4.0, 3.0, 0.0), (3.0, 1.0, 0.0));

  ctrlpoints3 : TPoints = (
        ( -3.0, 2.0, 0.0), ( 4.0, 1.0, 0.0),
        (2.0, -3.0, 0.0), (0.0, 0.0, 0.0));

type TPoints3D= array[0..3,0..3,0..2] of GLfloat;
var
   ctrlpoints3D : TPoints3D=  (
   ((-1.5, -1.5, 4.0), (-0.5, -1.5, 2.0),
    (0.5, -1.5, -1.0), (1.5, -1.5, 2.0)),
   ((-1.5, -0.5, 1.0), (-0.5, -0.5, 3.0),
    (0.5, -0.5, 0.0), (1.5, -0.5, -1.0)),
   ((-1.5, 0.5, 4.0), (-0.5, 0.5, 0.0),
    (0.5, 0.5, 3.0), (1.5, 0.5, 4.0)),
   ((-1.5, 1.5, -2.0), (-0.5, 1.5, -2.0),
    (0.5, 1.5, 0.0), (1.5, 1.5, -1.0))
);

   ambient : array[0..3] of GLfloat =(0.2, 0.2, 0.2, 1.0);
   position : array[0..3] of GLfloat =(0.0, 0.0, 2.0, 1.0);
   mat_diffuse :array[0..3] of GLfloat = (0.6, 0.6, 0.6, 1.0);
   mat_specular : array[0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
   mat_shininess : array[0..0] of GLfloat =( 50.0 );

procedure initLights();
begin
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);

   glLightfv(GL_LIGHT0, GL_AMBIENT, @ambient);
   glLightfv(GL_LIGHT0, GL_POSITION, @position);

   glMaterialfv(GL_FRONT, GL_DIFFUSE, @mat_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR, @mat_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, @mat_shininess);
end;

procedure initGL();
begin
   glClearColor(1.0, 1.0, 1.0, 0.0);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_MAP1_VERTEX_3);
   glEnable(GL_MAP2_VERTEX_3);
   glEnable(GL_AUTO_NORMAL); // for lighting
   glMapGrid2f(20, 0.0, 1.0, 20, 0.0, 1.0);
   initlights();
end;

procedure DrawBezierCurve(xTrans,yTrans : single;
                          ctrlpoints : TPoints );
var
   i : Integer;
begin
   glColor3f(0.0,0.0, 0.0);
   glPushMatrix();
     glTranslatef(xTrans,yTrans,0);
     glMap1f(GL_MAP1_VERTEX_3, 0.0, 1.0, 3, 4, @ctrlpoints);
     glBegin(GL_LINE_STRIP);
      for i := 0 to 30  do
         glEvalCoord1f(i/30.0);
     glEnd();
     //    The following code displays the control points as dots.
     glPointSize(5.0);
     glColor3f(Random, Random,0);
     glBegin(GL_POINTS);
      for i := 0 to  4-1 do
         glVertex3fv(@ctrlpoints[i,0]);
      glEnd();
   glPopMatrix();
   glFlush();
end;

procedure DrawBezierSurface(xTrans,yTrans,zTrans : single;
                            ctrlpoints : TPoints3D );
var
   i , j : Integer;
begin

glColor3f(0.0, 0.0, 0.0);

   glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4,
           0, 1, 12, 4, @ctrlpoints);

   glPushMatrix ();
   glTranslatef(xTrans,yTrans,zTrans);
   glRotatef(85.0, 1.0, 1.0, 1.0);
   glEvalMesh2(GL_FILL, 0, 20, 0, 20); // for Ligthing
    for j :=0 to 8 do
     begin
      glBegin(GL_LINE_STRIP);
        for i := 0 to  30 do
          glEvalCoord2f(i/30.0, j/8.0);
      glEnd();
      glBegin(GL_LINE_STRIP);
        for i := 0 to  30 do
          glEvalCoord2f(j/8.0, i/30.0);
      glEnd();
   end;
   glPopMatrix ();
   glFlush();

end;

procedure DrawGLScene();
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  DrawBezierCurve(0,0,ctrlpoints1);
  DrawBezierCurve(-6,0,ctrlpoints2);
//  DrawBezierCurve(3,-3,ctrlpoints3);

  DrawBezierSurface(0,0,0,ctrlpoints3D);

  swapBuffers(f_Hdc);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
 DrawGLScene();
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL();
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
  // Prevent A Divide By Zero If The Window Is Too Small
  if (Height=0)
    then Height:=1;
   glViewport(0, 0, width, Height );
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (width <= Height)  then
      glOrtho(-5.0, 5.0, -5.0*Height/width,
               5.0*Height/width, -5.0, 5.0)
   else
      glOrtho(-5.0*width/Height,
               5.0*width/Height, -5.0, 5.0, -5.0, 5.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

end.
