unit Ch13;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs ,OpenGL , SPF, ExtCtrls , StrUtils ;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    OpenDialog1: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  f_Hdc : LongInt;

implementation

{$R *.dfm}
var
  xrot : GLfloat;                 // Rotates Cube On The X Axis
  yrot : GLfloat;                 // Rotates Cube On The Y Axis
  fileName : string;
  Obj1  : GLuint;                 // Storage For The Display List
  x_Scale : single = 1 ;
  y_Scale : single = 1 ;
  z_Scale : single = 1 ;

procedure ReadRawDefaultFormat( fileName : string );
var
   f : text ;
   Ax,Ay,Az ,Bx,By,Bz, Cx,Cy,Cz : single;
   objectName : string;
begin
   assignfile(f,fileName);
   reset(f);

   readln(f,objectName);
   form1.Caption :=  objectName  ;

   while not eof(f) do
    begin
       readln(f,Ax,Ay,Az ,Bx,By,Bz, Cx,Cy,Cz);
       glBegin(GL_TRIANGLES);
         glColor3f(0,1,0);
         glVertex3f(ax,ay,az);
         glColor3f(1,1,0);
         glVertex3f(bx,by,bz);
         glColor3f(0,0,0);
         glVertex3f(cx,cy,cz);
       glEnd();
    end;
   glFlush();
   closefile(f);
end;

procedure ReadGemFormat( fileName : string );
var
   f : text ;
   Ax,Ay,Az :  array of single; // dynamic arrays
   i,j , NoOfPoints , NoOfFaces : LongInt;
   VertexesToLink, NoOfPolygonVertexes : array of LongInt; //dynamic arrays
begin
   assignfile(f,fileName);
   reset(f);

   readln(f,NoOfPoints,NoOfFaces);

   setlength(Ax,NoOfPoints);
   setlength(Ay,NoOfPoints);
   setlength(Az,NoOfPoints);
   setLength(NoOfPolygonVertexes,NoOfFaces);

   for i:=0 to High(Ax) do
       readln(f,Ax[i],Ay[i],Az[i]);

   for i:=0 to NoOfFaces-1 do
   begin
     read(f,NoOfPolygonVertexes[i]);
     setLength(VertexesToLink,NoOfPolygonVertexes[i]);

     glBegin(GL_POLYGON);
       for j:=0 to  NoOfPolygonVertexes[i]-1 do
        begin
            read(f,VertexesToLink[j]);
            glColor3f(Random,Random,0);
            glVertex3f(Ax[VertexesToLink[j]-1],
                       Ay[VertexesToLink[j]-1],
                       Az[VertexesToLink[j]-1]);
       end;
     glEnd();
   end;

   closefile(f);
end;

procedure initGL();
begin
   glClearColor(1.0, 1.0, 1.0, 0.0);
   glEnable(GL_DEPTH_TEST);
   glDepthFunc (GL_LESS);
   glClearDepth (1);
end;

procedure DrawGLScene();
begin
   glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

   glLoadIdentity();										// Reset The View
   glRotatef(xrot,1.0,0.0,0.0);
   glRotatef(yrot,0.0,1.0,0.0);
   glScalef(x_Scale,y_Scale,z_Scale);
   glCallList (Obj1); //efficient way !

   swapBuffers(f_Hdc);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
 DrawGLScene();
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 OpenDialog1.Filter :=
         'Raw Files (*.raw)|*.raw|'+
         'Geometery Files (*.Gem)|*.Gem|'+
         'All Files (*.*)|*.*';
 if  OpenDialog1.Execute  then  { Display Open dialog box }
     fileName := OpenDialog1.FileName { File selected in dialog }
 else
     halt(1);

 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL();

 Obj1 := glGenLists(1);
 glNewList (Obj1, GL_COMPILE);
 if LowerCase(RightStr(fileName,3))='gem' then
   ReadGemFormat(fileName);
 if LowerCase(RightStr(fileName,3))='raw' then
    ReadRawDefaultFormat(fileName);
 glEndList;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 glDeleteLists(Obj1,1);
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
  // Prevent A Divide By Zero If The Window Is Too Small
  if (Height=0)
    then Height:=1;
   glViewport(0, 0, width, Height );
   gluPerspective (30, width/ Height, 1, 100);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (width <= Height)  then
      glOrtho(-3.0, 3.0, -3.0*Height/width,
               3.0*Height/width, -3.0, 3.0)
   else
      glOrtho(-3.0*width/Height,
               3.0*width/Height, -3.0, 3.0, -3.0, 3.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  yrot := yrot - 5;
  xrot := xrot + 5;
  Application.ProcessMessages; //DoEvents
  InvalidateRect(Handle, nil, False);//   DrawGLScene();
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
if key = '+' then
  begin
    x_Scale := x_Scale + 0.05 ;
    y_Scale := y_Scale + 0.05 ;
    z_Scale := z_Scale + 0.05 ;
  end;

if key = '-' then
  begin
    x_Scale := x_Scale - 0.05  ;
    y_Scale := y_Scale - 0.05  ;
    z_Scale := z_Scale - 0.05  ;
  end;
end;

end.


