unit Ch14;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms,Dialogs ,OpenGL , SPF, ExtCtrls ;
type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
var
  Form1: TForm1;

  f_Hdc : LongInt;

implementation
{$R *.dfm}
var
  base : GLuint;   // Base Display List For The Font Set
  cnt1 : GLfloat;  // 1st Counter Used To Move Text & For Coloring
  cnt2 : GLfloat;  // 2nd Counter Used To Move Text & For Coloring
  h_DC : HDC;	   // Private GDI Device Context

procedure BuildFont;  // Build Our Bitmap Font
var font: HFONT;      // Windows Font ID
begin
  h_DC := Form1.Canvas.Handle ;
  base := glGenLists(96);     // Storage For 96 Characters
  font := CreateFont(-24,     // Height Of Font
		     0,	      // Width Of Font
		     0,	      // Angle Of Escapement
		     0,	      // Orientation Angle
		     FW_BOLD, // Font Weight
		     0,	      // Italic
		     0,	      // Underline
		     0,	      // Strikeout
		     ANSI_CHARSET,       // Character Set Identifier
		     OUT_TT_PRECIS,      // Output Precision
		     CLIP_DEFAULT_PRECIS,// Clipping Precision
		     ANTIALIASED_QUALITY,// Output Quality
		     FF_DONTCARE or DEFAULT_PITCH, // Family And Pitch
		     'Tahoma');		       	   // Font Name

  SelectObject(h_DC, font);   // Selects The Font We Want
  // Builds 96 Characters Starting At Character 32
  wglUseFontBitmaps(h_DC, 32, 96, base);
end;

procedure KillFont;     	     // Delete The Font
begin
  glDeleteLists(base, 96); 	     // Delete All 96 Characters
end;

procedure glPrint(text : pchar);     // Custom GL "Print" Routine
begin
  if (text = '') then   	     // If There's No Text
          Exit;			     // Do Nothing

  glPushAttrib(GL_LIST_BIT);	     // Pushes The Display List Bits
  glListBase(base - 32);	     // Sets The Base Character to 32
  // Draws The Display List Text
  glCallLists(length(text), GL_UNSIGNED_BYTE, text);
  glPopAttrib();	      		// Pops The Display List Bits
end;

// This Will Be Called Right After The GL Window Is Created
procedure InitGL(Width: GLsizei; Height: GLsizei);
var fWidth, fHeight  : GLfloat;
begin
  glClearColor(1.0, 1.0, 1.0, 0.0);// Clear The Background Color To white
  glClearDepth(1.0);		   // Enables Clearing Of The Depth Buffer
  glDepthFunc(GL_LESS);		   // The Type Of Depth Test To Do
  glEnable(GL_DEPTH_TEST);	   // Enables Depth Testing
  glShadeModel(GL_SMOOTH);	   // Enables Smooth Color Shading

  glMatrixMode(GL_PROJECTION);	   // Select The Projection Matrix
  glLoadIdentity();		   // Reset The Projection Matrix

  fWidth := Width;
  fHeight := Height;
  // Calculate The Aspect Ratio Of The Window
  gluPerspective(45.0, fWidth/fHeight,0.1,100.0);

  glMatrixMode(GL_MODELVIEW);	   // Select The Modelview Matrix

  BuildFont();			   // Build The Font
end;

procedure DrawGLScene();
begin
  // Clear The Screen And The Depth Buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();   		// Reset The View
  glTranslatef(0.0,0.0,-1.0);	// Move One Unit Into The Screen
  // Pulsing Colors Based On Text Position
  glColor3f(1.0*cos(cnt1),1.0*sin(cnt2),1.0-0.5*cos(cnt1+cnt2));
  // Position The Text On The Screen
  glRasterPos2f(-0.2+0.35*cos(cnt1), 0.35*(sin(cnt2)));
  glPrint('Win32 OpenGL Programming'); // Print GL Text To The Screen

  swapBuffers(f_Hdc);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  wglMakeCurrent(f_Hdc,hrc); //activate the RC
  // Prevent A Divide By Zero If The Window Is Too Small
  if (Height=0) then Height:=1;
  // Reset The Current Viewport And Perspective Transformation
  glViewport(0, 0, Width, Height);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  gluPerspective(45.0, Width/Height, 0.1, 100.0);
  glMatrixMode(GL_MODELVIEW);
  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 KillFont;
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle); 
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL(Width , Height);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  cnt1 := cnt1 + 0.1;					// Increase The First Counter
  cnt2 := cnt2 + 0.081;
  Application.ProcessMessages; //DoEvents
  InvalidateRect(Handle, nil, False);//   DrawGLScene();
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
 DrawGLScene();
end;

end.
