unit Ch16;
interface
uses
  Windows, Messages, SysUtils, Variants, Classes,
  Graphics, Controls, Forms, Dialogs, SPF,OpenGL, ExtCtrls ;
type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
var
  Form1: TForm1;

  f_Hdc : LongInt;

implementation
{$R *.dfm}
var
  g_FogDensity : single = 0.2;  // This holds how thick the fog is
  bFog : Boolean = true;	// This holds if we want fog ON of OFF
  rotY : single = 0.0;
  // We chose blue for our fog color.
  // Let's make the Fog Color BLUE
  fogColor : array[0..3] of single= (0.0,0.0,1.0,1.0);

///////////////// CREATE PYRAMID \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
///
///   This creates a pyramid with the center being (X, Y, Z).
///   The width and height depend on the WIDTH and HEIGHT passed in
///
///////////////// CREATE PYRAMID \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
procedure CreatePyramid( x,  y,  z : single; width, height : Integer);
begin
   glBegin(GL_TRIANGLES);
      // These vertices create the Back Side
      // Top point
      glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);
      // Bottom left point
      glColor3ub(0, 255, 255); glVertex3f(x - width, y - height, z - width);
      // Bottom right point
      glColor3ub(255, 0, 255); glVertex3f(x + width, y - height, z - width);

      // These vertices create the Front Side
      // Top point
      glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);
      // Bottom right point
      glColor3ub(0, 255, 255); glVertex3f(x + width, y - height, z + width);
      // Bottom left point
      glColor3ub(255, 0, 255); glVertex3f(x - width, y - height, z + width);

      // These vertices create the Front Left Side
      // Top point
      glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);
      // Front bottom point
      glColor3ub(255, 0, 255); glVertex3f(x - width, y - height, z + width);
      // Bottom back point
      glColor3ub(0, 255, 255); glVertex3f(x - width, y - height, z - width);

      // These vertices create the Front right Side
      // Top point
      glColor3ub(255, 0, 0);   glVertex3f(x, y + height, z);
      // Bottom back point
      glColor3ub(255, 0, 255); glVertex3f(x + width, y - height, z - width);
      // Front bottom point
      glColor3ub(0, 255, 255); glVertex3f(x + width, y - height, z + width);

   glEnd();

	// Now, when we want to render a different shape,
	// we just start a whole new glBegin() and glEnd();

    glBegin(GL_QUADS);
      // These vertices create the bottom of the pyramid
      // Front left point
      glColor3ub(0, 0, 255); glVertex3f(x - width, y - height, z + width);
      // Front right point
      glColor3ub(0, 0, 255); glVertex3f(x + width, y - height, z + width);
      // Back right point
      glColor3ub(0, 0, 255); glVertex3f(x + width, y - height, z - width);
      // Back left point
      glColor3ub(0, 0, 255); glVertex3f(x - width, y - height, z - width);
    glEnd();
end;

procedure RenderScene();
begin
// Clear The Screen And The Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();     // Reset The matrix
// This determines where the camera's position and view is
	gluLookAt(0, 0, 6,     0, 0, 0,     0, 1, 0);
// Rotate the pyramids along the Y axis
	glRotatef(rotY, 0.0, 1.0, 0.0);
// This creates a 3D pyramid in the center at (0, 0, 0)
	CreatePyramid(0, 0, 0, 1, 1);
// This creates a 3D pyramid back and to the right of the middle pyramid
	CreatePyramid(3, 0, -3, 1, 1);
// This creates a 3D pyramid back and to the left of the middle pyramid
	CreatePyramid(-3, 0, -3, 1, 1);
// This creates a 3D pyramid in right front of the camera
	CreatePyramid(0, 0, 5, 1, 1);
// Below we just increment our rotation degree by a small amount
// each frame. This increase the rotation of the cube along each axis.
  rotY :=rotY+ 2.6;  // Increment the Y rotation to increase the angle
// Swap the backbuffers to the foreground
  SwapBuffers(f_Hdc);
end;

procedure InitializeOpenGL();
begin
   glClearColor(0,0 , 1, 1);
   glEnable(GL_DEPTH_TEST); // Enables Depth Testing

   glFogi(GL_FOG_MODE, GL_EXP2);   	// Fog Mode
   // We pass in our array to set the fog color
   glFogfv(GL_FOG_COLOR, @fogColor);// Set Fog Color
   // The density is default 1.0,
   //but we will start with 0.2 (g_FogDensity).
   glFogf(GL_FOG_DENSITY, g_FogDensity);// How Dense Will The Fog Be
   // The "FOG_HINT" is how accurate the fog is calculate.
   // We don't really care,so we say so :)  GL_DONT_CARE let's openGL
   //choose either per vertex or per pixel fog.
   // Otherwise, we have the option to choose
   //GL_NICEST or GL_FASTEST if desired.
   glHint(GL_FOG_HINT, GL_DONT_CARE); // The Fog's calculation accuracy
   glFogf(GL_FOG_START, 0);  // Fog Start Depth
   glFogf(GL_FOG_END, 10.0); // Fog End Depth
   // To turn fog off, you can call glDisable(GL_FOG);
   glEnable(GL_FOG); // This enables our OpenGL Fog
end;

procedure TForm1.FormResize(Sender: TObject);
begin
    wglMakeCurrent(f_Hdc,hrc); //activate the RC
   if (height=0) then  // Prevent A Divide By Zero error
   	height:=1;    // Make the Height Equal One
   glViewport(0,0,width,height); // Make our viewport the whole window
   // We could make the view smaller inside
   // Our window if we wanted too.
   // The glViewport takes (x, y, width, height)
   // This basically means, what our our drawing boundries
   glMatrixMode(GL_PROJECTION);  // Select The Projection Matrix
   glLoadIdentity();	    // Reset The Projection Matrix
   // Calculate The Aspect Ratio Of The Window
   gluPerspective(45.0, width / height, 0.1, 150.0);
   glMatrixMode(GL_MODELVIEW);  // Select The Modelview Matrix
   glLoadIdentity();  	// Reset The Modelview Matrix
   InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitializeOpenGL();
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
 RenderScene;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Application.ProcessMessages; //DoEvents
  InvalidateRect(Handle, nil, False);//   DrawScene();
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin

if lowercase(key)='f'  then
  begin
      bFog := not bFog;	// This turns our bool to the opposite value
      if bFog then 	// If bFog is true
   	glEnable(GL_FOG)  // Turn on fog
      else
        glDisable(GL_FOG);   // Else turn OFF the fog
  end;

 if key='-' then
   begin
    // Decrease the fog density
    g_FogDensity := g_FogDensity - 0.015;
    // How Dense Will The Fog Be
    glFogf(GL_FOG_DENSITY, g_FogDensity);
    // Make sure we don't go below 0
    if(g_FogDensity < 0) then g_FogDensity := 0;
   end;

 if key='+' then
   begin
    g_FogDensity :=g_FogDensity+ 0.015;	 // Increase the fog density
    glFogf(GL_FOG_DENSITY, g_FogDensity); // How Dense Will The Fog Be
    // Make sure we don't go above 1
    if(g_FogDensity > 1) then g_FogDensity := 1;
   end;

  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

end.
