unit ch19;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes,
  Graphics, Controls, Forms,
  Dialogs , OpenGL , Textures , SPF, ExtCtrls ;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  f_Hdc : LongInt;

implementation

{$R *.dfm}

var
  xrot , yrot : GLfloat;
  xspeed : GLfloat;    	// X Rotation Speed
  yspeed : GLfloat;    	// Y Rotation Speed
  z : GLfloat = -10.0;  // Depth Into The Screen
  quadratic : GLUquadricObj; // Storage For Our Quadratic Objects
  LightAmbient : array [0..3] of GLfloat = (0.5, 0.5, 0.5, 1.0);
  LightDiffuse : array [0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
  LightPosition : array [0..3] of GLfloat = (0.0, 0.0, 2.0, 1.0);
  selectedobject : GLuint = 1;	 // Which Object To Draw
  // Textures
  MyTextureTex : glUint;

  procedure glBindTexture(target: GLenum; texture: GLuint);
  stdcall; external opengl32;

function InitGL:boolean;	// All Setup For OpenGL Goes Here

begin

  glEnable(GL_TEXTURE_2D);	   // Enable Texture Mapping
  glShadeModel(GL_SMOOTH);	   // Enable Smooth Shading
  glClearColor(0.0, 0.0, 0.0, 0.5);// Black Background
  glClearDepth(1.0);		   // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);	   // Enables Depth Testing
  glDepthFunc(GL_LEQUAL);	   // The Type Of Depth Testing To Do
  // Really Nice Perspective Calculations
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  LoadTexture('texture.jpg', MyTextureTex, FALSE);    // Load the Texture

  glLightfv(GL_LIGHT1, GL_AMBIENT, @LightAmbient); // Setup The Ambient Light
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @LightDiffuse); // Setup The Diffuse Light
  glLightfv(GL_LIGHT1, GL_POSITION,@LightPosition);// Position The Light
  glEnable(GL_LIGHT1);				   // Enable Light One
  // Create A Pointer To The Quadric Object (Return 0 If No Memory)
  quadratic := gluNewQuadric();
  gluQuadricNormals(quadratic, GLU_SMOOTH);// Create Smooth Normals
  gluQuadricTexture(quadratic, GL_TRUE);   // Create Texture Coords
  // Set The Texture Generation Mode For S To Sphere Mapping
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  // Set The Texture Generation Mode For T To Sphere Mapping
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);

  initGL:=true;   // Everything went fine
end;

procedure glDrawCube;
begin
  glBegin(GL_QUADS);
    // Front Face
    glNormal3f( 0.0, 0.0, 1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
    // Back Face
    glNormal3f( 0.0, 0.0,-1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
    // Top Face
    glNormal3f( 0.0, 1.0, 0.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0,  1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    // Bottom Face
    glNormal3f( 0.0,-1.0, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0, -1.0, -1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    // Right Face
    glNormal3f( 1.0, 0.0, 0.0);
    glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0,  1.0);
    // Left Face
    glNormal3f(-1.0, 0.0, 0.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
    glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0,  1.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0,  1.0);
    glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0);
  glEnd();
end;


function RenderScene():boolean;    // Here's Where We Do All The Drawing

begin
  // Clear The Screen And The Depth Buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();			 // Reset The View

  glTranslatef(0.0,0.0,z);
  glEnable(GL_TEXTURE_GEN_S); // Enable Texture Coord Generation For S
  glEnable(GL_TEXTURE_GEN_T); // Enable Texture Coord Generation For T
  // Bind the Texture to the object
  glBindTexture(GL_TEXTURE_2D, MyTextureTex);
  glPushMatrix();
  glRotatef(xrot,1.0,0.0,0.0);
  glRotatef(yrot,0.0,1.0,0.0);

  case selectedobject of
    0:    glDrawCube();
    1:    begin
            glTranslatef(0.0,0.0,-1.5);	  // Center The Cylinder
            // A Cylinder With A Radius Of 0.5 And A Height Of 2
            gluCylinder(quadratic,1.0,1.0,3.0,32,32);
          end;
          // Draw A Sphere With A Radius Of 1 And 16
          //Longitude And 16 Latitude Segments
    2:    gluSphere(quadratic,1.3,32,32);
    3:    begin
            glTranslatef(0.0,0.0,-1.5);			// Center The Cone
            // A Cone With A Bottom Radius Of .5 And A Height Of 2
            gluCylinder(quadratic,1.0,0.0,3.0,32,32);
          end;
  end;

  glPopMatrix();
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
  // Bind the Texture to the object
  glBindTexture(GL_TEXTURE_2D, MyTextureTex);

  glPushMatrix();
          glTranslatef(0.0, 0.0, -24.0);
          glBegin(GL_QUADS);
              glNormal3f( 0.0, 0.0, 1.0);
              glTexCoord2f(0.0, 0.0); glVertex3f(-13.3, -10.0,  10.0);
              glTexCoord2f(1.0, 0.0); glVertex3f( 13.3, -10.0,  10.0);
              glTexCoord2f(1.0, 1.0); glVertex3f( 13.3,  10.0,  10.0);
              glTexCoord2f(0.0, 1.0); glVertex3f(-13.3,  10.0,  10.0);
          glEnd();
  glPopMatrix();

  xrot := xrot + xspeed;  //increase the first counter
  yrot := yrot + yspeed;  //increase the second counter

  SwapBuffers(f_Hdc);

  RenderScene:=true;      // Everything Went OK
end;


procedure TForm1.FormResize(Sender: TObject);
begin
   wglMakeCurrent(f_Hdc,hrc); //activate the RC
  if (Height=0) then // Prevent A Divide By Zero If The Window Is Too Small
     Height:=1;      // By Making The Height One
  // Reset The Current Viewport And Perspective Transformation
  glViewport(0, 0, Width, Height);
  glMatrixMode(GL_PROJECTION);       // Select The Projection Matrix
  glLoadIdentity();                  // Reset The Projection Matrix
  // Calculate The Aspect Ratio Of The Window
  gluPerspective(45.0,Width/Height,0.1,100.0);
  glMatrixMode(GL_MODELVIEW);        // Select The Modelview Matrix
  glLoadIdentity;                     //Reset The Modelview Matrix
   InvalidateRect(Handle, nil, False);// DrawGLScene; Dr
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC(handle);
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 InitGL();
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
 RenderScene;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
   xspeed := xspeed + 0.5;
   yspeed := yspeed - 0.4;
   z := z + 0.02;
   xrot := xrot + 0.1;
   yrot := yrot + 0.3;
   InvalidateRect(Handle, nil, False);// DrawGLScene;

end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin
       selectedobject := selectedobject + 1;
       if (selectedobject>3) then selectedobject := 0;
       InvalidateRect(Handle, nil, False);// DrawGLScene;
end;

end.
