unit ch21;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs ,OpenGL,SPF, StdCtrls ;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
     procedure Idle(Sender: TObject; var Done: Boolean);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

type  displayLists=(
  RESERVED, BODY_SIDE, BODY_EDGE, BODY_WHOLE, ARM_SIDE,
  ARM_EDGE, ARM_WHOLE,  LEG_SIDE, LEG_EDGE, LEG_WHOLE,
  EYE_SIDE, EYE_EDGE, EYE_WHOLE );

 T3dArray = array [0..2] of GLfloat;
 TVertexf = T3DArray;  // An array of 3 float values.

var
 angle : GLfloat = 20;   //* in degrees */
 angle2 : GLfloat = 30;   //* in degrees */
 doubleBuffer :GLboolean = GL_TRUE;
 iconic :GLboolean= GL_FALSE;
 keepAspect :GLboolean = GL_FALSE;
 startx, starty : LongInt;
 moving : GLboolean = GL_FALSE;
 W :LongInt= 300;
 H :LongInt= 300 ;
 //* Initially, allow the artifacts. */
 useStencil : GLboolean = GL_FALSE;
 bodyWidth :GLdouble= 3.0;
 jump : single = 0.0;
//* *INDENT-OFF* */
  body : array of  TVertexf;
  arm : array of  TVertexf;
  leg : array of  TVertexf;
  eye : array of  TVertexf;
  lightZeroPosition : array[0..3] of GLfloat = (10.0, 14.0, 10.0, 1.0);
  lightZeroColor : array[0..3] of GLfloat= (0.8, 1.0, 0.8, 1.0);
  lightOnePosition : array[0..3] of GLfloat = (-1.0, 1.0, 1.0, 0.0);
  lightOneColor : array[0..3] of GLfloat = (0.6, 0.3, 0.2, 1.0);
  skinColor : array[0..3] of GLfloat = (0.1, 1.0, 0.1, 1.0);
  eyeColor : array[0..3] of GLfloat = (1.0, 0.2, 0.2, 1.0);
  lightZeroSwitch :GLboolean = GL_TRUE;
  lightOneSwitch :GLboolean= GL_TRUE;
  ElapsedTime, DemoStart, LastTime : DWord;
  wire: Boolean = FALSE;

  f_Hdc : longInt;

function Vertexf(x, y ,z: single): TVertexf;
begin
  // Utility function to build a vector:
  Vertexf[0] := x;
  Vertexf[1] := y;
  Vertexf[2] := z;
end;

procedure tessBegin(atype: GLenum); stdcall;
begin
  { This callback is invoked when the tesselator
    creates a new group of triangles. The atype
    parameter can be either GL_TRIANGLES, GL_TRIANGLE_FAN
    or GL_TRIANGLE_STRIP. }
  glBegin(atype);
end;

procedure tessEnd; stdcall;
begin
  // This callback is invoked when a group of triangles needs to be ended.
  glEnd;
end;

procedure tessVertex(data: Pointer); stdcall;
begin
  { This callback is invoked for each vertex
    in the triangulation. The data
    parameter is a pointer we passed to
    gluTessVertex(). (see below)
    In this case, it points to a float-precision 2D vector. }
  glVertex3fv(data);
end;

procedure InitVariables();
begin

 setlength(body,22);
 body[0]:=Vertexf(0,3,0);      body[1]:=Vertexf(1,1,0);
 body[2]:=Vertexf(5,1,0);      body[3]:=Vertexf(8,4,0);
 body[4]:=Vertexf(10,4,0);     body[5]:=Vertexf(11,5,0);
 body[6]:=Vertexf(11,11.5,0);  body[7]:=Vertexf(13,12,0);
 body[8]:=Vertexf(13,13,0);    body[9]:=Vertexf(10,13.5,0);
 body[10]:=Vertexf(13,14,0);   body[11]:=Vertexf(13,15,0);
 body[12]:=Vertexf(11,16,0);   body[13]:=Vertexf(8,16,0);
 body[14]:=Vertexf(7,15,0);    body[15]:=Vertexf(7,13,0);
 body[16]:=Vertexf(8,12,0);    body[17]:=Vertexf(7,11,0);
 body[18]:=Vertexf(6,6,0);     body[19]:=Vertexf(4,3,0);
 body[20]:=Vertexf(3,2,0);     body[21]:=Vertexf(1,2,0);

 setlength(arm,16);
 arm[0]:=Vertexf(8,10,0);     arm[1]:=Vertexf(9,9,0);
 arm[2]:=Vertexf(10,9,0);     arm[3]:=Vertexf(13,8,0);
 arm[4]:=Vertexf(14,9,0);     arm[5]:=Vertexf(16,9,0);
 arm[6]:=Vertexf(15,9.5,0);   arm[7]:=Vertexf(16,10,0);
 arm[8]:=Vertexf(15,10,0);    arm[9]:=Vertexf(15.5,11,0);
 arm[10]:=Vertexf(14.5,10,0); arm[11]:=Vertexf(14,11,0);
 arm[12]:=Vertexf(14,10,0);   arm[13]:=Vertexf(13,9,0);
 arm[14]:=Vertexf(11,11,0);   arm[15]:=Vertexf(9,11,0);

 setlength(leg,14);
 leg[0]:=Vertexf(8,6,0);     leg[1]:=Vertexf(8,4,0);
 leg[2]:=Vertexf(9,3,0);     leg[3]:=Vertexf(9,2,0);
 leg[4]:=Vertexf(8,1,0);     leg[5]:=Vertexf(8,0.5,0);
 leg[6]:=Vertexf(9,0,0);     leg[7]:=Vertexf(12,0,0);
 leg[8]:=Vertexf(10,1,0);    leg[9]:=Vertexf(10,2,0);
 leg[10]:=Vertexf(12,4,0);   leg[11]:=Vertexf(11,6,0);
 leg[12]:=Vertexf(10,7,0);   leg[13]:=Vertexf(9,7,0);

 setlength(eye,6);
 eye[0]:=Vertexf(8.75,15,0);     eye[1]:=Vertexf(9,14.7,0);
 eye[2]:=Vertexf(9.6,14.7,0);    eye[3]:=Vertexf(10.1,15,0);
 eye[4]:=Vertexf(9.6,15.25,0);   eye[5]:=Vertexf(9,15.25,0);

end;

procedure extrudeSolidFromPolygon(
     data : array of Tvertexf  ;  dataSize :GLuint;
    thickness :GLdouble; side, edge,  whole :GLuint);
var
   tobj : GLUtesselator;
   vertex :array[0..2] of  GLdouble;
   i,count :LongInt;

begin
   tobj :=nil;
   count := dataSize ;

  if (tobj = nil) then
    begin
     tobj := gluNewTess();  { create and initialize a GLU
                             polygon  tesselation object }
     
     gluTessCallback(tobj, GLU_TESS_BEGIN, @tessBegin);
     gluTessCallback(tobj, GLU_TESS_END, @tessEnd);
     gluTessCallback(tobj, GLU_TESS_VERTEX, @tessVertex);  
    end;

  glNewList(side, GL_COMPILE);
    glShadeModel(GL_SMOOTH);  { smooth minimizes seeing tessellation }
     gluTessBeginPolygon(tobj);
      gluTessBeginContour (tObj);
        for i := 0 to count do
         begin
          vertex[0] := data[i,0];
          vertex[1] := data[i,1];
          vertex[2] := 0;
          form1.Caption  := 'OpenGL Reflected Dinosaur!'
                             + floattostr(vertex[2]);
          gluTessVertex(tobj, @vertex, @data[i]);
        end;
      gluTessEndContour (tObj);
     gluTessEndPolygon(tobj);
  glEndList();

  glNewList(edge, GL_COMPILE);
// flat shade keeps angular hands from being "smoothed"
  glShadeModel(GL_FLAT);
  glBegin(GL_QUAD_STRIP);
  for i := 0 to count do
  begin
    // mod function handles closing the edge
    glVertex3f(data[i mod count,0], data[i mod count,1], 0.0);
    glVertex3f(data[i mod count,0], data[i mod count,1], thickness);
  end;

  glEnd();
  glEndList();

  glNewList(whole, GL_COMPILE);
  glFrontFace(GL_CW);
  glCallList(edge);
  glNormal3f(0.0, 0.0, -1.0);  //* constant normal for side */
  glCallList(side);
  glPushMatrix();
    glTranslatef(0.0, 0.0, thickness);
    glFrontFace(GL_CCW);
    glNormal3f(0.0, 0.0, 1.0);  // opposite normal for other side */
    glCallList(side);
  glPopMatrix();
  glEndList();
end;


procedure makeDinosaur();
begin
  extrudeSolidFromPolygon(body, 21, bodyWidth,
    ord(BODY_SIDE), ord(BODY_EDGE), ord(BODY_WHOLE));
  extrudeSolidFromPolygon(arm, 15, bodyWidth / 4,
    ord(ARM_SIDE), ord(ARM_EDGE),ord(ARM_WHOLE));
  extrudeSolidFromPolygon(leg, 13, bodyWidth / 2,
    ord(LEG_SIDE), ord(LEG_EDGE), ord(LEG_WHOLE));
  extrudeSolidFromPolygon(eye, 5, bodyWidth + 0.2,
    ord(EYE_SIDE), ord(EYE_EDGE), ord(EYE_WHOLE));
end;

procedure drawDinosaur();
begin
  glPushMatrix();
    glTranslatef(0.0, jump, 0.0);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, @skinColor);
    glCallList(ord(BODY_WHOLE));
    glPushMatrix();
     glTranslatef(0.0, 0.0, bodyWidth);
     glCallList(ord(ARM_WHOLE));
     glCallList(ord(LEG_WHOLE));
     glTranslatef(0.0, 0.0, -bodyWidth -(bodyWidth / 4));
     glCallList(ord(ARM_WHOLE));
     glTranslatef(0.0, 0.0, -bodyWidth / 4);
     glCallList(ord(LEG_WHOLE));
     glTranslatef(0.0, 0.0, (bodyWidth / 2) - 0.1);
     glMaterialfv(GL_FRONT, GL_DIFFUSE, @eyeColor);
     glCallList(ord(EYE_WHOLE));
    glPopMatrix();
  glPopMatrix();
end;

procedure drawFloor();
begin
  glDisable(GL_LIGHTING);
  glBegin(GL_QUADS);
    glVertex3f(-18.0, 0.0, 27.0);
    glVertex3f(27.0, 0.0, 27.0);
    glVertex3f(27.0, 0.0, -18.0);
    glVertex3f(-18.0, 0.0, -18.0);
  glEnd();
  glEnable(GL_LIGHTING);
end;

procedure redraw();
begin
  if (useStencil)then
     // Clear; default stencil clears to zero. */
    glClear(GL_COLOR_BUFFER_BIT or
            GL_DEPTH_BUFFER_BIT or GL_STENCIL_BUFFER_BIT)
   else
     // Not using stencil; just clear color and depth. */
     glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);

  glPushMatrix();
    // Perform scene rotations based on user mouse input. */
    glRotatef(angle2, 1.0, 0.0, 0.0);
    glRotatef(angle, 0.0, 1.0, 0.0);

    // Translate the dinosaur to be at (0,0,0). */
    glTranslatef(-8, -8, -bodyWidth / 2);

    glLightfv(GL_LIGHT0, GL_POSITION, @lightZeroPosition);
    glLightfv(GL_LIGHT1, GL_POSITION, @lightOnePosition);

    if (useStencil) then
    begin
      { We can eliminate the visual "artifact" of seeing the "flipped"
        dinosaur underneath the floor by using stencil.  The idea is
        draw the floor without color or depth update but so that
        a stencil value of one is where the floor will be.  Later when
        rendering the dinosaur reflection, we will only update pixels
        with a stencil value of 1 to make sure the reflection only
        lives on the floor, not below the floor. }

      // Don't update color or depth. */
      glDisable(GL_DEPTH_TEST);
      glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);

      // Draw 1 into the stencil buffer. */
      glEnable(GL_STENCIL_TEST);
      glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
      glStencilFunc(GL_ALWAYS, 1, $ffffffff);

      // Now render floor; floor pixels just get their stencil set to 1. */
      drawFloor();

      // Re-enable update of color and depth. */
      glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
      glEnable(GL_DEPTH_TEST);

      // Now, only render where stencil is set to 1. */
      glStencilFunc(GL_EQUAL, 1, $ffffffff);  // draw if ==1 */
      glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
    end;

    glPushMatrix();

      { The critical reflection step: Reflect dinosaur through the floor
         (the Y=0 plane) to make a relection. }
      glScalef(1.0, -1.0, 1.0);

      // Position lights now in reflected space. */
      glLightfv(GL_LIGHT0, GL_POSITION, @lightZeroPosition);
      glLightfv(GL_LIGHT1, GL_POSITION, @lightOnePosition);

      { XXX Ugh, unfortunately the back face culling reverses when we reflect
        the dinosaur.  Easy solution is just disable back face culling for
        rendering the reflection.  Also, the normals for lighting get screwed
        up by the scale; enabled normalize to ensure normals are still
        properly normalized despite the scaling.  We could have fixed the
        dinosaur rendering code, but this is more expedient. }
      glEnable(GL_NORMALIZE);
      glCullFace(GL_FRONT);

      // Draw the reflected dinosaur. */
      drawDinosaur();

      // Disable noramlize again and re-enable back face culling. */
      glDisable(GL_NORMALIZE);
      glCullFace(GL_BACK);

    glPopMatrix();

    // Restore light positions on returned from reflected space. */
    glLightfv(GL_LIGHT0, GL_POSITION, @lightZeroPosition);
    glLightfv(GL_LIGHT1, GL_POSITION, @lightOnePosition);

    if (useStencil) then
      { Don't want to be using stenciling for drawing the actual dinosaur
        (not its reflection) and the floor. }
      glDisable(GL_STENCIL_TEST);

    { Back face culling will get used to only draw either the top or the
       bottom floor.  This let's us get a floor with two distinct
       appearances.  The top floor surface is reflective and kind of red.
       The bottom floor surface is not reflective and blue. }

    // Draw "top" of floor.  Use blending to blend in reflection. */
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glColor4f(0.7, 0.0, 0.0, 0.3);
    drawFloor();
    glDisable(GL_BLEND);

    // Draw "bottom" of floor in blue. */
    glFrontFace(GL_CW);  // Switch face orientation. */
    glColor4f(0.1, 0.1, 0.7, 1.0);
    drawFloor();
    glFrontFace(GL_CCW);

    // Draw "actual" dinosaur, not its reflection. */
    drawDinosaur();

  glPopMatrix();

  SwapBuffers(f_Hdc);
end;

procedure controlLights(value : LongInt);
begin
   if value=1 then
   begin
    lightZeroSwitch := not(lightZeroSwitch);
    if (lightZeroSwitch) then
      glEnable(GL_LIGHT0)
     else
      glDisable(GL_LIGHT0);
   end;

   if value=2 then
   begin
    lightOneSwitch := not(lightOneSwitch);
    if (lightOneSwitch) then
      glEnable(GL_LIGHT1)
     else
      glDisable(GL_LIGHT1);
   end;

   if value=3 then
     useStencil := not(useStencil);
end;

procedure initGL();
begin
  InitVariables;
  redraw;
  makeDinosaur();
  glEnable(GL_CULL_FACE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  glMatrixMode(GL_PROJECTION);
  glEnable(GL_NORMALIZE);
  gluPerspective( { field of view in degree } 40.0,
   { aspect ratio } 1.0,
    { Z near } 1.0, { Z far } 80.0);
  glMatrixMode(GL_MODELVIEW);
  gluLookAt(0.0, 0.0, 40.0,  { eye is at (0,0,30) }
    0.0, 0.0, 0.0,      { center is at (0,0,0) }
    0.0, 1.0, 0.0);      { up is in postivie Y direction }
  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, @lightZeroColor);
  glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.1);
  glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.05);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, @lightOneColor);
  glEnable(GL_LIGHT0);

  glLightfv(GL_LIGHT0, GL_POSITION, @lightZeroPosition);
  glLightfv(GL_LIGHT1, GL_POSITION, @lightOnePosition);
  glViewport (0, 0, form1.Width, form1.Height);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC( form1.handle );
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 initGL();
 DemoStart :=GetTickCount;
 Application.OnIdle := Idle;
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button=mbLeft then
  begin
    moving := GL_TRUE;
    startx := x;
    starty := y;
   InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
  end;
end;

procedure TForm1.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    moving := GL_FALSE;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
   wglMakeCurrent(f_Hdc,hrc); //activate the RC
   glViewport( 0, 0, Width, Height);
   InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
  reDraw();                         // Draw the scene
end;

procedure TForm1.Idle(Sender: TObject; var Done: Boolean);
begin
if moving=GL_FALSE then
 begin
  LastTime :=ElapsedTime;
  ElapsedTime :=GetTickCount() - DemoStart;  // Calculate Elapsed Time
  // Average it out for smoother movement
  ElapsedTime :=(LastTime + ElapsedTime) DIV 2;
  jump :=  6.0 * abs(sin(ElapsedTime/8));
  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
 end;
end;

procedure TForm1.FormKeyPress(Sender: TObject; var Key: Char);
begin

  if Key = '4' then wire := not wire;
  if wire then glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
  else glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

  controlLights(strtoint(key));
  InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
end;

procedure TForm1.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if (moving) then
  begin
    angle := angle + (x - startx);
    angle2 := angle2 + (y - starty);
    startx := x;
    starty := y;
   InvalidateRect(Handle, nil, False);// DrawGLScene;  Draw the scene.
  end;
end;

end.
