unit Math_3D;

interface

uses
   Math;

const
 // Used to cover up the error in floating point
 MATCH_FACTOR : double = 0.9999;
 ZERO : integer = 0;

type
 VECTOR3D = record
        x, y, z : single ;
 end;

type
   V3D = array of VECTOR3D; 

type
  Cylinder=record
       Position : VECTOR3D;
       Axis : VECTOR3D;
       Radius : double;
  end;

type
  Plane=record
       Position : VECTOR3D;
       Normal : VECTOR3D;
 end;


type
   TMatrix33 = Record
     Mx : array[0..2] of array[0..2] of double;
 end;

type
    QUAD = record
	vVertices : V3D;
	vNormal : VECTOR3D;
	D : single;
    end;


function Vertexf(x0, y0 ,z0,
                 x1, y1 ,z1,
                 x2, y2 ,z2 : single): V3D;overload;

function Vertexf(x0, y0 ,z0,
                 x1, y1 ,z1,
                 x2, y2 ,z2,
                 x3, y3 ,z3 : single): V3D;overload;

 function Cross( vVector1,  vVector2 : VECTOR3D):VECTOR3D;
 function Vector( vPoint1,  vPoint2 : VECTOR3D):VECTOR3D;
 function Magnitude( vNormal :VECTOR3D):single;
 function Normalize( vNormal : VECTOR3D ):VECTOR3D;
 function Normal( vTriangle : V3D ):VECTOR3D;
 function PlaneDistance( Normal, Point :VECTOR3D): single;
 function IntersectedPlane( vTriangle , vLine : V3D ):boolean;
 function IntersectedPlane02( vPoly, vLine : V3D;
                             var vNormal : VECTOR3D ;
                             var originDistance : single):boolean;
 function Dot( vVector1, vVector2 :VECTOR3D ):single;
 function AngleBetweenVectors(Vector1, Vector2 :VECTOR3D ):double;
 function IntersectionPoint(vNormal : VECTOR3D ; vLine : V3D ;
                           distance : double ):VECTOR3D;
 function InsidePolygon( vIntersection : VECTOR3D ; Poly :V3D ;
                        verticeCount : LongInt ): boolean;
 function IntersectedPolygon( vPoly, vLine :V3D ;
                             verticeCount:integer ):boolean;

 function Matrix_making():TMatrix33;overload;
 function Matrix_making( Phi, Theta, Psi :double) : TMatrix33;overload;
 function Matrix_making( mx00, mx01, mx02,
		        mx10, mx11, mx12,
		        mx20, mx21, mx22 :double ): TMatrix33;overload;
 function Matrix_add(m1, m2 :TMatrix33 ):TMatrix33;
 function Matrix_subtract(m1,m2 :TMatrix33 ):TMatrix33;
 function Matrix_multiply( m1 , m2 : TMatrix33):TMatrix33;overload;
 function Matrix_multiply(
               m1 :TMatrix33; scale : double):TMatrix33;overload;
 function Matrix_multiply(
                  m1 : TMatrix33; v :VECTOR3D):VECTOR3D;overload;
 function Matrix_determinant(m : TMatrix33):double;             
 function Matrix_transpose(m : TMatrix33):TMatrix33;
 function Matrix_inverse(m1 : TMatrix33 ):TMatrix33;

 function Vector_add( v1 , v2 : VECTOR3D) : VECTOR3D;
 function Vector_subtract(v1,v2 : VECTOR3D) : VECTOR3D;
 function Vector_multiply(v1 : VECTOR3D; scale : double ): VECTOR3D;

 function Vector_Making(x, y ,z : single):VECTOR3D;

implementation

function Vector_Making(x, y ,z : single):VECTOR3D;
begin
     Vector_Making.x := x;
     Vector_Making.y := y;
     Vector_Making.z := z;
end;

function Vertexf(x0, y0 ,z0,
                 x1, y1 ,z1,
                 x2, y2 ,z2 : single): V3D;overload;
var
   res : V3D;
begin
  // Utility function to build a vectors:

  setLength(res,3);

  res[0].x := x0;
  res[0].y := y0;
  res[0].z := z0;

  res[1].x := x1;
  res[1].y := y1;
  res[1].z := z1;

  res[2].x := x2;
  res[2].y := y2;
  res[2].z := z2;

  Vertexf := res;
end;


function Vertexf(x0, y0 ,z0,
                 x1, y1 ,z1,
                 x2, y2 ,z2,
                 x3, y3 ,z3 : single): V3D;overload;
var
   res : V3D;
begin
  // Utility function to build a vectors:

  setLength(res,4);


  res[0].x := x0;
  res[0].y := y0;
  res[0].z := z0;

  res[1].x := x1;
  res[1].y := y1;
  res[1].z := z1;

  res[2].x := x2;
  res[2].y := y2;
  res[2].z := z2;

  res[3].x := x3;
  res[3].y := y3;
  res[3].z := z3;

  Vertexf := res;
end;


// The most important part of this tutorial
//is the plane equation, Ax + By + Cz + D = 0.

/////////////////////// CROSS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns a perpendicular vector from 2 given
/////   vectors by taking the cross product.
/////////////////////// CROSS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function Cross( vVector1,  vVector2 : VECTOR3D):VECTOR3D;
var
  // The vector to hold the cross product
  vNormal : VECTOR3D ;
begin
  // Once again, if we are given 2 vectors
  // (directions of 2 sides of a polygon)
  // then we have a plane define.  The cross
  // product finds a vector that is perpendicular
  // to that plane, which means it's point straight
  // out of the plane at a 90 degree angle.

  // The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													// Get the X value
  vNormal.x := ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y));

  // The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y := ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z));

  // The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z := ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x));

  Cross := vNormal;										// Return the cross product (Direction the polygon is facing - Normal)
end;

/////////////////////// VECTOR \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns a vector between 2 points
/////////////////////// VECTOR \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function Vector( vPoint1,  vPoint2 : VECTOR3D):VECTOR3D;
var
  vVector : VECTOR3D;
begin
   // In order to get a vector from 2 points (a direction) we need to
   // subtract the second point from the first point.

   vVector.x := vPoint1.x - vPoint2.x;  // Get the X value of our new vector
   vVector.y := vPoint1.y - vPoint2.y;  // Get the Y value of our new vector
   vVector.z := vPoint1.z - vPoint2.z;  // Get the Z value of our new vector

   Vector := vVector;										// Return our new vector
end;

/////////////////// MAGNITUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns the magnitude of a normal (or any other vector)
//////////////////// MAGNITUDE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function Magnitude( vNormal :VECTOR3D):single;
begin
 // This will give us the magnitude or "Norm" as some say, of our normal.
 // Here is the equation:
 //  magnitude = sqrt(V.x^2 + V.y^2 + V.z^2)  Where V is the vector

	Magnitude := sqrt( (vNormal.x * vNormal.x) +
                           (vNormal.y * vNormal.y) +
                           (vNormal.z * vNormal.z) );
end;

//////////////////// NORMALIZE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns a normalize vector (A vector exactly of length 1)
///////////////////// NORMALIZE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function Normalize( vNormal : VECTOR3D ):VECTOR3D;
var
    m_magnitude : single;
begin
         // Get the magnitude of our normal
	 m_magnitude := Magnitude(vNormal);

	// Now that we have the magnitude, we can
        // divide our normal by that magnitude.
	// That will make our normal a total length of 1.
        // This makes it easier to work with too.

        // Divide the X value of our normal by it's magnitude
	vNormal.x :=vNormal.x/ m_magnitude;
        // Divide the Y value of our normal by it's magnitude
	vNormal.y :=vNormal.y/ m_magnitude;
        // Divide the Z value of our normal by it's magnitude
	vNormal.z :=vNormal.z/ m_magnitude;

	// Finally, return our normalized normal.

	Normalize := vNormal;										// Return the new normal of length 1.
end;

////////////////////// NORMAL \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns the normal of a polygon
////    (The direction the polygon is facing)
/////////////////////// NORMAL \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function Normal( vTriangle : V3D ):VECTOR3D;
var
   vVector1 , vVector2 , vNormal : VECTOR3D;

begin
    // Get 2 vectors from the polygon (2 sides), Remember the order!
    vVector1 := Vector(vTriangle[2], vTriangle[0]);
    vVector2 := Vector(vTriangle[1], vTriangle[0]);
    // Take the cross product of our 2 vectors
    // to get a perpendicular vector
    vNormal  := Cross(vVector1, vVector2);

    // Now we have a normal, but it's at a
    // strange length, so let's make it length 1.
    // Use our function we created to normalize
    // the normal (Makes it a length of one)
    vNormal := Normalize(vNormal);

    Normal := vNormal; // Return our normal at our desired length
end;

////////////////// PLANE DISTANCE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns the distance between a plane and the origin
////////////////// PLANE DISTANCE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function PlaneDistance( Normal, Point :VECTOR3D): single;
 var
     distance : single ;
    // This variable holds the distance
    // from the plane tot he origin
begin

	//  D = -(Ax + By + Cz)
														// Basically, the negated dot product of the normal of the plane and the point. (More about the dot product in another tutorial)
	distance := - ((Normal.x * Point.x) +
                       (Normal.y * Point.y) +
                       (Normal.z * Point.z));

	PlaneDistance := distance;									// Return the distance
end;

////////////// INTERSECTED PLANE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This checks to see if a line intersects a plane
////////////// INTERSECTED PLANE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function IntersectedPlane( vTriangle , vLine : V3D ):boolean;
var
    distance1, distance2 , originDistance : single ;
    vNormal : VECTOR3D;
begin

        // We need to get the normal of our plane to go any further
	vNormal := Normal(vTriangle);

	originDistance := PlaneDistance(vNormal, vTriangle[0]);

	// Get the distance from point1 from the plane using:
        // Ax + By + Cz + D = (The distance from the plane)

	distance1 := ((vNormal.x * vLine[0].x)  +  // Ax +
		      (vNormal.y * vLine[0].y)  +  // Bx +
   		      (vNormal.z * vLine[0].z)) + originDistance; // Cz + D

	distance2 := ((vNormal.x * vLine[1].x)  + // Ax +
		      (vNormal.y * vLine[1].y)  + // Bx +
		      (vNormal.z * vLine[1].z)) + originDistance; // Cz + D
 // Check to see if both point's distances are both negative or both positive
	if(distance1 * distance2 >= 0)	then
        // Return false if each point has the same sign.
        // -1 and 1 would mean each point is on either side of the plane.
        // -1 -2 or 3 4 wouldn't...
	   IntersectedPlane := false
        else
           // The line intersected the plane, Return TRUE
           IntersectedPlane := true;
end;

///////////////// INTERSECTED PLANE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This checks to see if a line intersects a plane
///////////////// INTERSECTED PLANE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function IntersectedPlane02( vPoly, vLine : V3D;
                             var vNormal : VECTOR3D ;
                             var originDistance : single):boolean;
var
    distance1, distance2 : single ;
begin
        // We need to get the normal of our plane to go any further
	vNormal := Normal(vPoly);

	originDistance := PlaneDistance(vNormal, vPoly[0]);

	distance1 := ((vNormal.x * vLine[0].x)  + // Ax +
		      (vNormal.y * vLine[0].y)  + // Bx +
		      (vNormal.z * vLine[0].z)) + originDistance; // Cz + D

	distance2 := ((vNormal.x * vLine[1].x)  + // Ax +
		      (vNormal.y * vLine[1].y)  + // Bx +
		      (vNormal.z * vLine[1].z)) + originDistance;// Cz + D

// Check to see if both point's distances are both negative or both positive
	if(distance1 * distance2 >= 0) then
         // Return false if each point has the same sign.
         //  -1 and 1 would mean each point is on either side of the plane.
         //  -1 -2 or 3 4 wouldn't...
	  IntersectedPlane02 := false
        else
          // The line intersected the plane, Return TRUE
	   IntersectedPlane02 := true;

end;


//////////////////////////////// DOT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This computers the dot product of 2 vectors
///////////////////////////////// DOT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function Dot( vVector1, vVector2 :VECTOR3D ):single;
begin
  // The dot product is this equation:
  // V1.V2 = (V1.x * V2.x  +  V1.y * V2.y  +  V1.z * V2.z)
  // In math terms, it looks like this:  V1.V2 = ||V1|| ||V2|| cos(theta)
  // The '.' means DOT.   The || || is magnitude.

	Dot := ( (vVector1.x * vVector2.x) +
                 (vVector1.y * vVector2.y) +
                 (vVector1.z * vVector2.z) );
end;


////////////// ANGLE BETWEEN VECTORS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This checks to see if a point is inside the ranges of a polygon
////////////// ANGLE BETWEEN VECTORS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function AngleBetweenVectors(Vector1, Vector2 :VECTOR3D ):double;
var
   dotProduct ,vectorsMagnitude : single;
begin
	// Get the dot product of the vectors
	 dotProduct := Dot(Vector1, Vector2);

	// Get the product of both of the vectors magnitudes
	 vectorsMagnitude := Magnitude(Vector1) * Magnitude(Vector2) ;

	// Return the arc cosine of the
        //(dotProduct / vectorsMagnitude) which is the angle in RADIANS.
	AngleBetweenVectors := ( arccos( dotProduct / vectorsMagnitude ) );
end;

////////////// INTERSECTION POINT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This returns the intersection point
////    of the line that intersects the plane
/////////////// INTERSECTION POINT \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function IntersectionPoint(vNormal : VECTOR3D ; vLine : V3D ;
                           distance : double ):VECTOR3D;
var
    vPoint , vLineDir : VECTOR3D;
    Numerator , Denominator , dist : double ;
begin
	//  We need to find the 3D point that is actually
	// on the plane.  Here are some steps to do that:

	vLineDir := Vector(vLine[1], vLine[0]);	// Get the Vector of the line
	vLineDir := Normalize(vLineDir);       	// Normalize the lines vector
        // Use the plane equation with the normal and the line
	Numerator := - (vNormal.x * vLine[0].x +
		        vNormal.y * vLine[0].y +
		        vNormal.z * vLine[0].z + distance);

 // Get the dot product of the line's vector and the normal of the plane
	Denominator := Dot(vNormal, vLineDir);

        // Check so we don't divide by zero
	if( Denominator = 0.0)	 then
          begin
                // Return an arbitrary point on the line
		IntersectionPoint := vLine[0];
                exit;
          end;

        // Divide to get the multiplying (percentage) factor
	dist := Numerator / Denominator;

	vPoint.x := (vLine[0].x + (vLineDir.x * dist));
	vPoint.y := (vLine[0].y + (vLineDir.y * dist));
	vPoint.z := (vLine[0].z + (vLineDir.z * dist));

	IntersectionPoint := vPoint;								// Return the intersection point
end;


//////////////////// INSIDE POLYGON \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This checks to see if a point is inside the ranges of a polygon
//////////////////// INSIDE POLYGON \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function InsidePolygon( vIntersection : VECTOR3D ; Poly :V3D ;
                        verticeCount : LongInt ): boolean;
var
   Angle : double;
   vA, vB : VECTOR3D; // Create temp vectors
   i : integer;
begin
        Angle := 0;
        // Go in a circle to each vertex and get the angle between
	for  i := 0 to verticeCount-1 do
	begin
                // Subtract the intersection point from the current vertex
		vA := Vector(Poly[i], vIntersection);
		// Subtract the point from the next vertex
		vB := Vector(Poly[(i + 1) mod verticeCount], vIntersection);
// Find the angle between the 2 vectors and add them all up as we go along
		Angle := Angle+ AngleBetweenVectors(vA, vB);
	end;

        // If the angle is greater than 2 PI, (360 degrees)
	if(Angle >= (MATCH_FACTOR * (2.0 * PI)) )  then
		InsidePolygon := TRUE  // The point is inside of the polygon
        else
 	        InsidePolygon := FALSE;								// If you get here, it obviously wasn't inside the polygon, so Return FALSE
end;


/////////////// INTERSECTED POLYGON \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
/////	This checks if a line is intersecting a polygon
//////////////// INTERSECTED POLYGON \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

function IntersectedPolygon( vPoly, vLine :V3D ;
                             verticeCount:integer ):boolean;
var
  vNormal , vIntersection : VECTOR3D;
  originDistance : single ;
begin
	if(not(IntersectedPlane02(vPoly, vLine,
               vNormal,   originDistance))) then
          begin
		IntersectedPolygon := false;
                exit;
          end;

	vIntersection := IntersectionPoint(vNormal,
                                           vLine, originDistance);

	if(InsidePolygon(vIntersection, vPoly, verticeCount)) then
          begin
		IntersectedPolygon := true;
                exit;
          end;

	// If we get here, we must have NOT collided
	IntersectedPolygon := false;								// There was no collision, so return false
end;


function Matrix_making():TMatrix33;overload;
var
   res : TMatrix33;
begin
	res.Mx[0][0]:=1.0; res.Mx[0][1]:=0.0; res.Mx[0][2]:=0.0;

	res.Mx[1][0]:=0.0; res.Mx[1][1]:=1.0; res.Mx[1][2]:=0.0;
	res.Mx[2][0]:=0.0; res.Mx[2][1]:=0.0; res.Mx[2][2]:=1.0;
        Matrix_making := res;
end;


function Matrix_making( mx00, mx01, mx02,

		        mx10, mx11, mx12,
		        mx20, mx21, mx22 :double ): TMatrix33;overload;
var
   res : TMatrix33;
begin
	res.Mx[0][0]:=mx00; res.Mx[0][1]:=mx01; res.Mx[0][2]:=mx02;

	res.Mx[1][0]:=mx10; res.Mx[1][1]:=mx11; res.Mx[1][2]:=mx12;
	res.Mx[2][0]:=mx20; res.Mx[2][1]:=mx21; res.Mx[2][2]:=mx22;
        Matrix_making := res;
end;

function Matrix_making( Phi, Theta, Psi :double) : TMatrix33;overload;

  var

     c1 ,s1 , c2, s2, c3, s3 :double;

     res : TMatrix33;

begin
	 c1:=cos(Phi);   s1:=sin(Phi);    c2:=cos(Theta);

         s2:=sin(Theta); c3:=cos(Psi);    s3:=sin(Psi);

	res.Mx[0][0]:=c2*c3;

	res.Mx[0][1]:=-c2*s3;
	res.Mx[0][2]:=s2;
	res.Mx[1][0]:=s1*s2*c3+c1*s3;
	res.Mx[1][1]:=-s1*s2*s3+c1*c3;
	res.Mx[1][2]:=-s1*c2;
	res.Mx[2][0]:=-c1*s2*c3+s1*s3;
	res.Mx[2][1]:=c1*s2*s3+s1*c3;
	res.Mx[2][2]:=c1*c2;

        Matrix_making := res;

end;


function Matrix_add(m1, m2 :TMatrix33 ):TMatrix33;
var
    res : TMatrix33;
begin
	res.Mx[0][0] := m1.Mx[0][0] + m2.Mx[0][0];
	res.Mx[0][1] := m1.Mx[0][1] + m2.Mx[0][1];
	res.Mx[0][2] := m1.Mx[0][2] + m2.Mx[0][2];
	res.Mx[1][0] := m1.Mx[1][0] + m2.Mx[1][0];
	res.Mx[1][1] := m1.Mx[1][1] + m2.Mx[1][1];
	res.Mx[1][2] := m1.Mx[1][2] + m2.Mx[1][2];
	res.Mx[2][0] := m1.Mx[2][0] + m2.Mx[2][0];
	res.Mx[2][1] := m1.Mx[2][1] + m2.Mx[2][1];
	res.Mx[2][2] := m1.Mx[2][2] + m2.Mx[2][2];

	Matrix_add := res;

end;

function Matrix_subtract(m1,m2 :TMatrix33 ):TMatrix33;
var
    res : TMatrix33;
begin
	res.Mx[0][0] := m1.Mx[0][0] - m2.Mx[0][0];

	res.Mx[0][1] := m1.Mx[0][1] - m2.Mx[0][1];
	res.Mx[0][2] := m1.Mx[0][2] - m2.Mx[0][2];
	res.Mx[1][0] := m1.Mx[1][0] - m2.Mx[1][0];
	res.Mx[1][1] := m1.Mx[1][1] - m2.Mx[1][1];
	res.Mx[1][2] := m1.Mx[1][2] - m2.Mx[1][2];
	res.Mx[2][0] := m1.Mx[2][0] - m2.Mx[2][0];
	res.Mx[2][1] := m1.Mx[2][1] - m2.Mx[2][1];
	res.Mx[2][2] := m1.Mx[2][2] - m2.Mx[2][2];

	Matrix_subtract := res;

end;

function Matrix_multiply( m1 , m2 : TMatrix33):TMatrix33;overload;
var
    res : TMatrix33;
begin
	res.Mx[0][0] := m1.Mx[0][0]*m2.Mx[0][0] +
                        m1.Mx[0][1]*m2.Mx[1][0] +
                        m1.Mx[0][2]*m2.Mx[2][0];
	res.Mx[1][0] := m1.Mx[1][0]*m2.Mx[0][0] +
                        m1.Mx[1][1]*m2.Mx[1][0] +
                        m1.Mx[1][2]*m2.Mx[2][0];
	res.Mx[2][0] := m1.Mx[2][0]*m2.Mx[0][0] +
                        m1.Mx[2][1]*m2.Mx[1][0] +
                        m1.Mx[2][2]*m2.Mx[2][0];
	res.Mx[0][1] := m1.Mx[0][0]*m2.Mx[0][1] +
                        m1.Mx[0][1]*m2.Mx[1][1] +
                        m1.Mx[0][2]*m2.Mx[2][1];
	res.Mx[1][1] := m1.Mx[1][0]*m2.Mx[0][1] +
                        m1.Mx[1][1]*m2.Mx[1][1] +
                        m1.Mx[1][2]*m2.Mx[2][1];
	res.Mx[2][1] := m1.Mx[2][0]*m2.Mx[0][1] +
                        m1.Mx[2][1]*m2.Mx[1][1] +
                        m1.Mx[2][2]*m2.Mx[2][1];
	res.Mx[0][2] := m1.Mx[0][0]*m2.Mx[0][2] +
                        m1.Mx[0][1]*m2.Mx[1][2] +
                        m1.Mx[0][2]*m2.Mx[2][2];
	res.Mx[1][2] := m1.Mx[1][0]*m2.Mx[0][2] +
                        m1.Mx[1][1]*m2.Mx[1][2] +
                        m1.Mx[1][2]*m2.Mx[2][2];
	res.Mx[2][2] := m1.Mx[2][0]*m2.Mx[0][2] +
                        m1.Mx[2][1]*m2.Mx[1][2] +
                        m1.Mx[2][2]*m2.Mx[2][2];
	Matrix_multiply := res;
end;


function Matrix_multiply(
               m1 :TMatrix33; scale : double):TMatrix33;overload;
var
   res : TMatrix33;

begin
	res.Mx[0][0] := m1.Mx[0][0] * scale;

	res.Mx[0][1] := m1.Mx[0][1] * scale;
	res.Mx[0][2] := m1.Mx[0][2] * scale;
	res.Mx[1][0] := m1.Mx[1][0] * scale;
	res.Mx[1][1] := m1.Mx[1][1] * scale;
	res.Mx[1][2] := m1.Mx[1][2] * scale;
	res.Mx[2][0] := m1.Mx[2][0] * scale;
	res.Mx[2][1] := m1.Mx[2][1] * scale;
	res.Mx[2][2] := m1.Mx[2][2] * scale;

	Matrix_multiply := res;

end;

function Matrix_multiply(
                  m1 : TMatrix33; v :VECTOR3D):VECTOR3D;overload;
var
   res : VECTOR3D;
begin


  res.x := m1.Mx[0][0]*v.X + m1.Mx[0][1]*v.Y + m1.Mx[0][2]*v.Z;

  res.y := m1.Mx[1][0]*v.X + m1.Mx[1][1]*v.Y + m1.Mx[1][2]*v.Z;
  res.z := m1.Mx[2][0]*v.X + m1.Mx[2][1]*v.Y + m1.Mx[2][2]*v.Z;

  Matrix_multiply := res;

end;

function Matrix_determinant(m : TMatrix33):double;
begin
    Matrix_determinant :=

      m.Mx[0][0]*(m.Mx[1][1]*m.Mx[2][2]-m.Mx[1][2]*m.Mx[2][1])

      - m.Mx[0][1]*(m.Mx[1][0]*m.Mx[2][2]-m.Mx[1][2]*m.Mx[2][0])
      + m.Mx[0][2]*(m.Mx[1][0]*m.Mx[2][1]-m.Mx[1][1]*m.Mx[2][0]);
end;

function Matrix_transpose(m : TMatrix33):TMatrix33;
var
   t : double;
begin
	t:= m.Mx[0][2]; m.Mx[0][2] := m.Mx[2][0]; m.Mx[2][0] := t;

	t := m.Mx[0][1]; m.Mx[0][1] := m.Mx[1][0]; m.Mx[1][0] := t;

	t := m.Mx[1][2]; m.Mx[1][2] := m.Mx[2][1]; m.Mx[2][1] := t;

	Matrix_transpose :=m;

end;



function Matrix_inverse(m1 : TMatrix33 ):TMatrix33;

var

   det : double;

   res : TMatrix33;

begin
  det := Matrix_determinant(m1);

 res.Mx[0][0] := m1.Mx[1][1]*m1.Mx[2][2] - m1.Mx[1][2]*m1.Mx[2][1];

 res.Mx[0][1] := m1.Mx[2][1]*m1.Mx[0][2] - m1.Mx[2][2]*m1.Mx[0][1];
 res.Mx[0][2] := m1.Mx[0][1]*m1.Mx[1][2] - m1.Mx[0][2]*m1.Mx[1][1];
 res.Mx[1][0] := m1.Mx[1][2]*m1.Mx[2][0] - m1.Mx[1][0]*m1.Mx[2][2];
 res.Mx[1][1] := m1.Mx[2][2]*m1.Mx[0][0] - m1.Mx[2][0]*m1.Mx[0][2];
 res.Mx[1][2] := m1.Mx[0][2]*m1.Mx[1][0] - m1.Mx[0][0]*m1.Mx[1][2];
 res.Mx[2][0] := m1.Mx[1][0]*m1.Mx[2][1] - m1.Mx[1][1]*m1.Mx[2][0];
 res.Mx[2][1] := m1.Mx[2][0]*m1.Mx[0][1] - m1.Mx[2][1]*m1.Mx[0][0];
 res.Mx[2][2] := m1.Mx[0][0]*m1.Mx[1][1] - m1.Mx[0][1]*m1.Mx[1][0];

 Matrix_inverse := Matrix_multiply(res, 1.0/det);

end;


function Vector_add( v1 , v2 : VECTOR3D) : VECTOR3D;

var
  res : VECTOR3D;
begin
		res.x := v1.x + v2.x;

		res.y := v1.y + v2.y;
		res.z := v1.z + v2.z;
      Vector_add := res;
end;

function Vector_subtract(v1,v2 : VECTOR3D) : VECTOR3D;

var
  res : VECTOR3D;
begin
		res.x := v1.x - v2.x;

		res.y := v1.y - v2.y;
		res.z := v1.z - v2.z;
    Vector_subtract := res;
end;


function Vector_multiply(v1 : VECTOR3D; scale : double ): VECTOR3D;

var
  res : VECTOR3D;
begin
		res.x := v1.x * scale;

		res.y := v1.y * scale;
		res.z := v1.z * scale;
        Vector_multiply := res;
end;


end.
