unit ch25;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs , OpenGL , SPF , Math_3D;

type
  TForm1 = class(TForm)
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { Private declarations }

  public
    { Public declarations }
  end;
                   
var
  Form1: TForm1;

implementation

{$R *.dfm}

var

 vTriangle : V3D ;
// This is our line that we will be checking
//against the polygon's plane for collision.
// We position the line going directly through the polygon at first.
 vLine : V3D;

 f_Hdc : longInt;

procedure initGL();
begin
 vTriangle := Math_3D.Vertexf(-1,0,0,  0,1,0,   1,0,0);
 vLine :=  Math_3D.Vertexf(0,0.5,-0.5,  0,0.5,0.5 ,0,0,0);
end;

procedure RenderScene( m_mode : integer );
var
   bCollided : boolean;
begin
        bCollided := false;

        // Clear The Screen And The Depth Buffer
        glClearColor(1,1,1,0);
	glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();   // Reset The matrix

	// Let's set our camera to the left a bit for a better view
        // This determines where the camera's position and view is
     	gluLookAt(-2.5, 0.5, 0.5,  0, 0.5, 0,   0, 1, 0);

	// Below we give OpenGL the 3 vertices of
        //our triangle.  Once again, we put them
	// into an array of VECTOR3D structures so
        //we could dynamically move it around screen.

    	glBegin (GL_TRIANGLES);	// This is our BEGIN to draw
	   glColor3ub(255, 0, 0);
	   glVertex3f(vTriangle[0].x, vTriangle[0].y, vTriangle[0].z);

	   glColor3ub(255, 255, 0);
	   glVertex3f(vTriangle[1].x, vTriangle[1].y, vTriangle[1].z);

	   glColor3ub(0, 255, 255);
	   glVertex3f(vTriangle[2].x, vTriangle[2].y, vTriangle[2].z);
	glEnd();     // This is the END of drawing

     if m_mode=0 then
       begin
// Below we use our function we just wrote to see if the plane of the
// triangle and the line intersect. It will return true if that's the case.

         bCollided := IntersectedPlane(vTriangle, vLine);
       end;

     if m_mode=1 then
       begin
// Now, instead of just testing against the plane, we take it a step further
// and test if we actually hit the polygon.  This is a more usable collision.
// We give our function the polygon, the line to test with, and the
// number of vertices of our polygon

	bCollided := IntersectedPolygon(vTriangle, vLine, 3);
      end;

// Below we draw the line that the polygon will be colliding with.
// We will check to see if the line collides with the polygons
// plane, and if it does,
// we will turn the line green to show when it is intersecting the plane.

	glBegin (GL_LINES);  	// This is our BEGIN to draw
     // If we collided, change the color of the line to illustrate this.
		if(bCollided) then
          // Make the line RED if we collided with the triangle's plane
			glColor3ub(255, 0, 0)
		else
                        // Make the line blue if we didn't collide
			glColor3ub(0, 0, 25);

                // Let's draw the normal centered on the triangle
		glVertex3fv(@vLine[0]);
               // Draw the normal of the polygon from the
               //center of the polygon to better see it
		glVertex3fv(@vLine[1]);
	glEnd();   // This is the END of drawing

	// That's it, now use the LEFT and RIGHT arrow keys to
        //move it around to further see it in action.

	SwapBuffers(f_Hdc);// Swap the backbuffers to the foreground

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 f_Hdc := GetDC( form1.handle );
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 initGL();
end;

procedure TForm1.FormResize(Sender: TObject);
begin
   wglMakeCurrent(f_Hdc,hrc); //activate the RC

	if (height=0) then  // Prevent A Divide By Zero error
	begin
		height:=1;  // Make the Height Equal One
	end;

	glViewport(0,0,width,height);// Make our viewport the whole window

       	glMatrixMode(GL_PROJECTION); // Select The Projection Matrix
	glLoadIdentity();   // Reset The Projection Matrix

	gluPerspective(45.0,width/height,0.1,150.0);

	glMatrixMode(GL_MODELVIEW); // Select The Modelview Matrix
	glLoadIdentity();	    // Reset The Modelview Matrix

   InvalidateRect(Handle, nil, False);// Draw the scene.

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
  RenderScene(1);           // Draw the scene
end;


procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
     case key of
	  VK_ESCAPE: // Check if we hit the ESCAPE key.
  	    PostQuitMessage(0);	 // Tell windows we want to quit

	  VK_UP:  // Check if we hit the UP ARROW key.
                 begin
                   // Move the left point of the triangle to the left
		   vTriangle[0].x :=vTriangle[0].x + 0.01;
                   // Move the top point of the triangle to the left
		   vTriangle[1].x := vTriangle[1].x + 0.01;
                   // Move the right point of the triangle to the left
		   vTriangle[2].x :=vTriangle[2].x + 0.01;
                   // Redraw the scene to reflect the new position
                   RenderScene(1);
                  end;
	 VK_DOWN:  // Check if we hit the DOWN ARROW key.
                  begin
		   vTriangle[0].x :=vTriangle[0].x - 0.01;
		   vTriangle[1].x :=vTriangle[1].x - 0.01;
		   vTriangle[2].x :=vTriangle[2].x - 0.01;
		   RenderScene(1);
                  end;
	 VK_LEFT: // Check if we hit the LEFT ARROW key.
                 begin
		  vTriangle[0].z :=vTriangle[0].z - 0.01;
		  vTriangle[1].z :=vTriangle[1].z - 0.01;
		  vTriangle[2].z :=vTriangle[2].z - 0.01;
		  RenderScene(1);
                 end;
	 VK_RIGHT:// Check if we hit the RIGHT ARROW key.
                 begin
		  vTriangle[0].z :=vTriangle[0].z + 0.01;
		  vTriangle[1].z :=vTriangle[1].z + 0.01;
		  vTriangle[2].z :=vTriangle[2].z + 0.01;
                  RenderScene(1);
                end;
	 VK_PRIOR:// Check if we hit the PAGE UP key.
                 begin
		  vTriangle[0].y := vTriangle[0].y+ 0.01;
		  vTriangle[1].y := vTriangle[1].y + 0.01;
		  vTriangle[2].y := vTriangle[2].y +0.01;
		  RenderScene(1);
                 end;
	VK_NEXT:  // Check if we hit the PAGE DOWN key.
                begin
		  vTriangle[0].y := vTriangle[0].y - 0.01;
		  vTriangle[1].y := vTriangle[1].y - 0.01;
		  vTriangle[2].y := vTriangle[2].y - 0.01;
		  RenderScene(1);
                end;
    end;

end;


end.
