unit ch27;

interface

uses
  Windows, Messages, Classes, Graphics, Controls,
  Forms,Dialogs, OpenGL ,SPF , glExt ;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }

       procedure Idle(Sender: TObject; var Done: Boolean);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

var

  ElapsedTime : Integer;      // Elapsed time between frames

  // Other vaiables
  TorusDL : glUint;
  SeperateSpecular : Boolean; // if Seperate Specular is Enabled
  Segments : Integer;         // quality of torus

  // Lights
  LightPos : Array [0..3] of GLfloat = (0.0, 0.0, 0.0, 1.0);
  MatDiffuse : Array [0..3] of GLfloat = (0.8, 0.8, 0.8, 1.0);
  MatSpecular: Array [0..3] of GLfloat = (1.0, 1.0, 1.0, 1.0);
  MatShine   : GLfloat = 50.0;

  f_Hdc : longInt;

  DemoStart, LastTime : Integer;

{------------------------------------------------------------------}
{  Function to convert int to string. (No sysutils = smaller EXE)  }
{------------------------------------------------------------------}
// using SysUtils increase file size by 100K
function IntToStr(Num : Integer) : String;
begin
  Str(Num, result);
end;

{------------------------------------------------------------------}
{  Function to draw a torus                                        }
{------------------------------------------------------------------}
procedure CreateTorus(TubeRadius, Radius : GLfloat;
                      Sides, Rings : Integer);
var I, J : Integer;
    theta, phi, theta1 : GLfloat;
    cosTheta, sinTheta : GLfloat;
    cosTheta1, sinTheta1 : GLfloat;
    ringDelta, sideDelta : GLfloat;
    cosPhi, sinPhi, dist : GLfloat;
begin
  sideDelta := 2.0 * Pi / Sides;
  ringDelta := 2.0 * Pi / rings;
  theta := 0.0;
  cosTheta := 1.0;
  sinTheta := 0.0;

  TorusDL :=glGenLists(1);
  glNewList(TorusDL, GL_COMPILE);
    for i := rings - 1 downto 0 do
    begin
      theta1 := theta + ringDelta;
      cosTheta1 := cos(theta1);
      sinTheta1 := sin(theta1);
      glBegin(GL_QUAD_STRIP);
        phi := 0.0;
        for j := Sides downto 0 do
        begin
          phi := phi + sideDelta;
          cosPhi := cos(phi);
          sinPhi := sin(phi);
          dist := Radius + (TubeRadius * cosPhi);

          glNormal3f(cosTheta1 * cosPhi,
                      -sinTheta1 * cosPhi, sinPhi);
          glVertex3f(cosTheta1 * dist, -sinTheta1 * dist,
                     TubeRadius * sinPhi);

          glNormal3f(cosTheta * cosPhi,
                     -sinTheta * cosPhi, sinPhi);
          glVertex3f(cosTheta * dist,
                      -sinTheta * dist, TubeRadius * sinPhi);
        end;
      glEnd();
      theta := theta1;
      cosTheta := cosTheta1;
      sinTheta := sinTheta1;
    end;
  glEndList();
end;

{------------------------------------------------------------------}
{  Function to draw the actual scene                               }
{------------------------------------------------------------------}
procedure glDraw();
begin
  // Clear The Screen And The Depth Buffer
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity();          // Reset The View
  glTranslatef(0.0,0.0,-14);

  glRotatef(ElapsedTime/20, 1, 0, 0);
  glRotatef(ElapsedTime/16, 0, 1, 0);

  glCallList(TorusDL);

  SwapBuffers(f_Hdc);
end;


{------------------------------------------------------------------}
{  Initialise OpenGL                                               }
{------------------------------------------------------------------}
procedure glInit();
begin
  glClearColor(0.0, 0.0, 0.0, 0.0);   // Black Background
  glShadeModel(GL_SMOOTH);            // Enables Smooth Color Shading
  glClearDepth(1.0);                  // Depth Buffer Setup
  glEnable(GL_DEPTH_TEST);            // Enable Depth Testing
  glDepthFunc(GL_LESS);		      // The Type Of Depth Test To Do

  //Realy Nice perspective calculations
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

  // Turn on OpenGL lighting
  glEnable(GL_LIGHTING);

  // Create a light
  glLightfv(GL_LIGHT0, GL_POSITION, @LightPos);
  glEnable(GL_LIGHT0);

  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, @MatDiffuse);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, @MatSpecular);
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, @MatShine);

  SeperateSpecular :=FALSE;
  Segments :=64;
  CreateTorus(1, 3, Segments, Segments);
end;

{------------------------------------------------------------------}
{  Handle window resize                                            }
{------------------------------------------------------------------}
procedure glResizeWnd(Width, Height : Integer);
begin
  if (Height = 0) then     // prevent divide by zero exception
    Height := 1;
  // Set the viewport for the OpenGL window
  glViewport(0, 0, Width, Height);
  glMatrixMode(GL_PROJECTION);      // Change Matrix Mode to Projection
  glLoadIdentity();                 // Reset View
  // Do the perspective calculations. Last value = max clipping depth
  gluPerspective(45.0, Width/Height, 1.0, 100.0);

  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
  glLoadIdentity();                   // Reset View
end;

procedure determine_hardware_acceleration(
                   Handle: HDC;ColorBits,DepthBufferBits:integer);

var
  generic_format : integer;
  generic_accelerated : integer;

  pfd: TPixelFormatDescriptor;
  nPixelFormat: Integer;
  pfd_new : PIXELFORMATDESCRIPTOR ;
begin

  FillChar(pfd, SizeOf(pfd), 0);

  with pfd do begin
    nSize     := sizeof(pfd);             // Size of this structure
    nVersion  := 1;                       // Version number
    dwFlags   := PFD_SUPPORT_OPENGL Or PFD_DRAW_TO_WINDOW
                 Or  PFD_TYPE_RGBA
                 or PFD_DOUBLEBUFFER {to avoid flickering} ; // Flags
    iPixelType:= PFD_TYPE_RGBA;           // RGBA pixel values
    cColorBits:= ColorBits;               // 24-bit color
    cDepthBits:= DepthBufferBits;         // 32-bit depth buffer
    iLayerType:= PFD_MAIN_PLANE;          // Layer type
  end;

  nPixelFormat := ChoosePixelFormat(Handle, @pfd);

  DescribePixelFormat (Handle, npixelFormat,
                       sizeof(PIXELFORMATDESCRIPTOR), pfd_new);

     generic_format :=
                  pfd_new.dwFlags and PFD_GENERIC_FORMAT;

     generic_accelerated :=
                  pfd_new.dwFlags and PFD_GENERIC_ACCELERATED;

     if (generic_format=1 and not(generic_accelerated)) then
     begin
      MessageBox(0,'softdware','3DInfo',
                           MB_OK+MB_ICONINFORMATION);
     end;

     if (generic_format=1 and generic_accelerated) then
     begin
      MessageBox(0,'hardware - MCD','3DInfo',
                           MB_OK+MB_ICONINFORMATION);
     end;

     if ( not(generic_format)=1 and not(generic_accelerated)) then
     begin
       MessageBox(0,'hardware - ICD','3DInfo',
                           MB_OK+MB_ICONINFORMATION);
     end;
end;


procedure glInfo();
var
   vendor,renderer,version,extension : pchar;
begin
	vendor := pchar (glGetString(GL_VENDOR));
	renderer := pchar ( glGetString(GL_RENDERER));
	version := pchar (glGetString(GL_VERSION));
	extension := pchar (glGetString(GL_EXTENSIONS));
        MessageBox(0,pchar('vendor: '+vendor+#13#10+
                   'renderer: '+renderer+#13#10+
                   'version: '+version+#13#10+
                   'extension: '+extension),'glInfo',
                   MB_OK+MB_ICONINFORMATION);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin

 DemoStart := GetTickCount();   // Get Time when demo started
 f_Hdc := GetDC( handle );
 SetDCPixelFormat(f_Hdc,16,16);// Create a rendering context.
 glInit();
 Application.OnIdle := Idle;
 determine_hardware_acceleration(f_Hdc,16,16);
 glInfo();

end;

procedure TForm1.FormResize(Sender: TObject);
begin

    wglMakeCurrent(f_Hdc,hrc); //activate the RC
      glResizeWnd(width,height);
    InvalidateRect(Handle, nil, False);// Draw the scene.

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 CleanUp(f_Hdc);// Clean up and terminate.
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
 wglMakeCurrent(f_Hdc,hrc); //activate the RC
  glDraw() // Draw the scene.
end;

procedure TForm1.Idle(Sender: TObject; var Done: Boolean);
begin

               
  LastTime :=ElapsedTime;

   // Calculate Elapsed Time
  ElapsedTime :=GetTickCount() - DemoStart;

  // Average it out for smoother movement
  ElapsedTime := (LastTime + ElapsedTime) DIV 2;

  InvalidateRect(Handle, nil, False);//  Draw the scene.

end;


procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

  // Turn on specular highlights if supported
  if (key = ord('N')) then
  begin
    if SeperateSpecular then  // If its enabled, then disable
      glLightModel(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR)
    else
    begin
      if isSupported('GL_EXT_separate_specular_color') then
        glLightModel(GL_LIGHT_MODEL_COLOR_CONTROL,
                     GL_SEPARATE_SPECULAR_COLOR)
      else
        glLightModel(GL_LIGHT_MODEL_COLOR_CONTROL,
                     GL_SINGLE_COLOR);
    end;
    SeperateSpecular :=NOT(SeperateSpecular);
  end;

  if key=VK_ADD then
  begin
    Segments :=Segments*2;
    CreateTorus(1, 3, Segments, Segments);
  end;

  if key=VK_SUBTRACT then
  begin
    Segments :=Segments DIV 2;
    if Segments < 4 then
      Segments :=4;
    CreateTorus(1, 3, Segments, Segments);
  end;

     InvalidateRect(Handle, nil, False);//  Draw the scene.

end;

end.
